/* ************************************************************************
*   File: boards.c                                      Part of CircleMUD *
*  Usage: handling of multiple bulletin boards                            *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

#include <dirent.h>
 
#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "db.h"
#include "boards.h"
#include "interpreter.h"
#include "handler.h"
#include "improved-edit.h"

/* Notice: This is not the stock circle board system. Nor is it the
 * slightly revamped version via OasisOLC. This is loosely based on
 * Patrick Dughi's dynamic board system, with a few minor updates.
 * Contact Patrick <dughi@imaxx.net> for copyright info. -spl 11/18/00
 */
 
struct board_info_type *board_info;
extern struct descriptor_data *descriptor_list;
extern struct room_data *world;
extern struct index_data *obj_index;
 
void Board_init_boards(void) {
  int n,o, board_vnum;
  struct dirent **namelist;

  n = scandir(BOARD_DIRECTORY, &namelist, 0, alphasort);

  if (n < 0) {
    log("SYSERR: No board files found! (?)\n");
    return;
  }
 
  for (o=0; o != n; o++) {
    if(strcmp("..",namelist[o]->d_name) && strcmp(".",namelist[o]->d_name)) {
      sscanf(namelist[o]->d_name,"%d", &board_vnum);
      create_new_board(board_vnum);
	}
  }

  /* just before we close, lets look at boards huh? */
  look_at_boards(); 
}
 
struct board_info_type * create_new_board(int board_vnum) {
           extern struct board_info_type *board_info;
           struct board_msg_info *bmsg;

           int boards=0, t_messages=0, error=0, t[4], mnum, poster,timestamp,sflag;
           char buf[512];
           FILE *fl;
           struct board_info_type *temp_board=NULL, *old_board;
           struct obj_data *obj;
           struct board_memory_type *memboard, *list;

           sprintf(buf,"%s/%d",BOARD_DIRECTORY,board_vnum);

           if(!(fl = fopen(buf, "r"))) {

/* this is for boards which exist as an object, but don't have a file yet */
                /* Ie, new boards */

               if(!(fl = fopen(buf,"w"))) {
                 log("Hm. Error while creating new board file. Most likely, the etc/boards folder does not exist.");
               }
               else {
	     
                 obj = read_object(real_object(board_vnum), REAL);

                 CREATE(temp_board, struct board_info_type, 1);
  	         READ_LVL(temp_board)=GET_OBJ_VAL(obj, 0);
  	         WRITE_LVL(temp_board)=GET_OBJ_VAL(obj, 1);
                 REMOVE_LVL(temp_board)=GET_OBJ_VAL(obj, 2);
                 BOARD_VNUM(temp_board)=board_vnum;
	         BOARD_MNUM(temp_board)=0;
                 BOARD_NEXT(temp_board)=NULL;
                 BOARD_MESSAGES(temp_board)=NULL;
                 extract_obj(obj);
                 fprintf(fl,"Board File\n%d %d %d %d\n",READ_LVL(temp_board),
     WRITE_LVL(temp_board), REMOVE_LVL(temp_board), BOARD_MNUM(temp_board));
                 fclose(fl);
            
	        old_board=board_info;
                board_info=temp_board;
                BOARD_NEXT(temp_board)=old_board;
                return temp_board;

               }
           }
           else {
             get_line(fl,buf);
             if((strcasecmp("Board File", buf))) {
             }
             else {
               boards++;
               CREATE(temp_board, struct board_info_type, 1);
               temp_board->vnum=board_vnum;
               get_line(fl, buf); 
               if (sscanf(buf,"%d", t) != 1) {
                 error=1;
               }
               if (!(real_object(board_vnum))) {
                 sprintf(buf, "Erasing non-existant board file: %d",
								board_vnum);
		 sprintf(buf,"%s/%d",BOARD_DIRECTORY,board_vnum);
                 unlink(buf);
                 if(temp_board) 
  		   free(temp_board);
                 return NULL;    /* realize why I do this, yes? */
               }
               obj = read_object(real_object(board_vnum),REAL);

                 READ_LVL(temp_board)=GET_OBJ_VAL(obj, 0);
                 WRITE_LVL(temp_board)=GET_OBJ_VAL(obj, 1);
                 REMOVE_LVL(temp_board)=GET_OBJ_VAL(obj, 2);	
		 BOARD_MNUM(temp_board)=t[0];
                 BOARD_NEXT(temp_board)=NULL;
                 BOARD_MESSAGES(temp_board)=NULL;
                 extract_obj(obj);

               if (error) {
                 sprintf(buf,"Parse error in board %d - attempting to continue\n",
  		   BOARD_VNUM(temp_board));
               /* attempting board ressurection */

               if(!(t[0] > 0 && t[0] < LVL_IMPL)) {
                 READ_LVL(temp_board) = LVL_IMMORT;
               }
               if(!(t[1] > 0 && t[1] < LVL_IMPL)) {
                       WRITE_LVL(temp_board) = LVL_IMMORT;
               }
               if(!(t[2] > 0 && t[2] < LVL_IMPL)) {
                 REMOVE_LVL(temp_board) = LVL_IMPL;
               }
               if(!(t[3] > 0 && t[3] < LVL_IMPL)) {
                /*  This is an indication of a very unfun board */
                /* I guess if I ever optimize any of this raggedy code */
		/* this will actually mean something. Right now, its nada */
                       BOARD_MNUM(temp_board) = -1;       
                   }
		 }
                 list=NULL; /* or BOARD_MEMORY(temp_board) but this should
				be null anyway...*/
	         while(get_line(fl,buf)) {
                   /* this is the loop that reads the messages 
			and also the message read info stuff */
                   if (*buf == 'S') {
	/* Note, we assume that all messages are read in before the hash
					memory lists */
                     sscanf(buf,"S %d %d %d ", &mnum, &poster, &timestamp);
                     CREATE(memboard, struct board_memory_type, 1);
                     MEMORY_READER(memboard)=poster;
                     MEMORY_TIMESTAMP(memboard)=timestamp;

		     bmsg=BOARD_MESSAGES(temp_board);
		     sflag=0;
			/* check and see if memory is still valid */

		     while(bmsg && !sflag) {

 		       if(MESG_TIMESTAMP(bmsg) ==MEMORY_TIMESTAMP(memboard)
			&& (mnum == ((MESG_TIMESTAMP(bmsg)%301 +
				MESG_POSTER(bmsg)%301)%301))) {
			   sflag=1;
			  /* probably ought to put a flag saying that
			     memory should be deleted because the reader
			     no longer exists on the mud, but it will
			     figure it out in a few iterations. Let it be 
			     happy, and let me be lazy */
		       }
		       bmsg=MESG_NEXT(bmsg);
                    }
            if(sflag) {
                       if(BOARD_MEMORY(temp_board,mnum)) {
                         list=BOARD_MEMORY(temp_board,mnum);
                         BOARD_MEMORY(temp_board,mnum)=memboard;
                         MEMORY_NEXT(memboard)=list;
                       }
                       else {
                         BOARD_MEMORY(temp_board,mnum)=memboard;
                         MEMORY_NEXT(memboard)=NULL;
                       }
                    } else {
		/* memory no longer in use */
			free(memboard);
	             }
                   } else if (*buf == '#') {
                     t_messages++;
                     parse_message(fl, temp_board);
                   }
                 }
                BOARD_MNUM(temp_board)=t_messages;

/* add it to the list. */

                old_board=board_info;
		board_info=temp_board;
                BOARD_NEXT(temp_board)=old_board;

               }
           fclose(fl);
             }
  /* hold on. Lets think about this:
        Board messages are added in the folowing way - 1,2,3,4,5...etc
        However, when they're read in, its fifo, 5,4,3,2,1, so between
        each reboot, the existing order is reversed.  hm. */
  reverse_message_order(temp_board);

  return temp_board;
}


  
void reverse_message_order(struct board_info_type *t_board) {
  struct board_msg_info *old,*new,*store;
   
  old=BOARD_MESSAGES(t_board);
  
  if(!old) 
    return;
  
  store=new=NULL;
 
  /* yes, there's a better way for doing this, but I think I had some
     problems with second thoughts after accidently deleting stuff */
  while(old) {
	store=new;
    CREATE(new, struct board_msg_info, 1);
    MESG_POSTER(new)=MESG_POSTER(old);
    MESG_TIMESTAMP(new)=MESG_TIMESTAMP(old);
    MESG_SUBJECT(new)=MESG_SUBJECT(old);
    MESG_DATA(new)=MESG_DATA(old);
    MESG_NEXT(new)=store;

    store=old;
    old=MESG_NEXT(old);
    free(store);
  }

  BOARD_MESSAGES(t_board)=new;
}
  
void parse_message( FILE *fl, struct board_info_type *t_board) {
  struct board_msg_info *tmsg, *t2msg;
  char subject[81];
  char buf[8192];
  
  CREATE(tmsg, struct board_msg_info, 1);
  
  fscanf(fl, "%ld\n", &(MESG_POSTER(tmsg)));
  fscanf(fl, "%ld\n", &(MESG_TIMESTAMP(tmsg)));
 
  get_line(fl,subject);

  MESG_SUBJECT(tmsg)=strdup(subject);
  MESG_DATA(tmsg)=fread_string(fl,buf);
  MESG_NEXT(tmsg)=NULL;

  t2msg=BOARD_MESSAGES(t_board);
  BOARD_MESSAGES(t_board)=tmsg;
  MESG_NEXT(tmsg) = t2msg;
}
       
void look_at_boards() {
  int counter, messages=0;
  struct board_info_type *tboard=board_info;
  struct board_msg_info *msg;
     
  for(counter=0;tboard;counter++) {
     msg=BOARD_MESSAGES(tboard);
     while(msg) {
       messages++;
       msg=MESG_NEXT(msg);
     }
     tboard=BOARD_NEXT(tboard);
  }

  log("There are %d boards located; %d messages",counter, messages);
}
  
int Board_show_board(int board_vnum, struct char_data *ch) {
  struct board_info_type *thisboard=board_info;
  struct board_msg_info *message;
  char *tmstr;
  int msgcount=0,yesno=0;
  char buf[MAX_STRING_LENGTH], buf2[MAX_STRING_LENGTH], buf3[MAX_STRING_LENGTH];
  char name[127];
 				/* board locate */
  if(IS_NPC(ch)) {
    send_to_char(ch, "Gosh.. now .. if only mobs could read.. you'd be doing good.\r\n");
    return 1;
  }

  while(thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard=BOARD_NEXT(thisboard);
  }

  if (!thisboard) {
    log("Creating new board - board #%d", board_vnum);
    thisboard=create_new_board(board_vnum);
  }
 
  if (GET_LEVEL(ch) < READ_LVL(thisboard)) {
    send_to_char(ch, "You try but fail to understand the holy words.\r\n");
    return 1;
  }
  /* send the standard board boilerplate */
 
  sprintf(buf2,"This is a bulletin board.\r\n"
    "Usage:READ/DELETE <messg #>, RESPOND <messg #>, WRITE <header>.\r\n");
  if (!BOARD_MNUM(thisboard) || !BOARD_MESSAGES(thisboard)) {
    strcat(buf2, "The board is empty.\r\n");
    send_to_char(ch, buf2);
    return 1;
  } else 
    sprintf(buf2, "%sThere %s %d %s on the board.\r\n", buf2, 
		(BOARD_MNUM(thisboard) == 1) ? "is" : "are",BOARD_MNUM(thisboard),
		(BOARD_MNUM(thisboard) == 1) ? "message" : "messages");
  message=BOARD_MESSAGES(thisboard);

  for (message=BOARD_MESSAGES(thisboard);message;message=MESG_NEXT(message)) {
    tmstr = (char *) asctime(localtime( &MESG_TIMESTAMP(message)));
    *(tmstr + strlen(tmstr) - 1) = '\0';
    yesno=mesglookup(message,ch,thisboard);
  
    sprintf(name,"%s", get_name_by_id(MESG_POSTER(message)) ? 
		get_name_by_id(MESG_POSTER(message)) : "Unknown");
    sprintf(buf3,"(%s)",CAP(name));

    if (MESG_SUBJECT(message) && yesno) {
      sprintf(buf,"&n[&yx&n] &w(&y%2d&w)&n : %6.10s %-23s :: &y%s&n \r\n", ++msgcount,tmstr,
      buf3, MESG_SUBJECT(message) ? MESG_SUBJECT(message) : "No Subject");
      strcat(buf2,buf);
    } else {
      sprintf(buf,"&n[ ] &w(&y%2d&w)&n : %6.10s %-23s :: &y%s&n \r\n", ++msgcount, tmstr,
            buf3, MESG_SUBJECT(message) ? MESG_SUBJECT(message) : "No Subject");
            strcat(buf2,buf);
	}
  }
  page_string(ch->desc, buf2, 1);
  return 1;
}
 
 
int Board_display_msg(int board_vnum, struct char_data * ch, int arg)
{
  extern struct board_info_type *board_info; 
  struct board_info_type *thisboard=board_info;
  struct board_msg_info *message;
  char *tmstr;
  int msgcount,mem,sflag;
  char name[127], buf[MAX_STRING_LENGTH];
  struct board_memory_type *mboard_type, *list;
 
  if(IS_NPC(ch)) {
    send_to_char(ch, "Silly mob - reading is for pcs!\r\n");
    return 1;
  }
 
  /* guess we'll have to locate the board now in the list */

  while(thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard=BOARD_NEXT(thisboard);
  }

  if (!thisboard) {
    log("Creating new board - board #%d", board_vnum);
    thisboard=create_new_board(board_vnum);
  } 
 
  if (GET_LEVEL(ch) < READ_LVL(thisboard)) {
    send_to_char(ch, "You try but fail to understand the holy words.\r\n");
    return 1;
  }
    
  if (!BOARD_MESSAGES(thisboard)) {
    send_to_char(ch, "The board is empty!\r\n");
    return 1;
  }

  /* now we locate the message.*/  
  message=BOARD_MESSAGES(thisboard);
  if (arg < 1) {
    send_to_char(ch, "You must specify the (positive) number of the message to be read!\r\n");
    return 1;
  }

  for(msgcount=arg;message && msgcount!=1;msgcount--) 
    message=MESG_NEXT(message);

  if(!message) {
    send_to_char(ch, "That message exists only in your imagination.\r\n");
    return 1;
  }
  /* Have message, let's add the fact that this player read the mesg */
 
  mem = ((MESG_TIMESTAMP(message)%301 + MESG_POSTER(message)%301)%301);

  /*make the new node */
  CREATE(mboard_type, struct board_memory_type, 1);
  MEMORY_READER(mboard_type)=GET_IDNUM(ch);
  MEMORY_TIMESTAMP(mboard_type)=MESG_TIMESTAMP(message);
  MEMORY_NEXT(mboard_type)=NULL;

  /* Let's make sure that we don't already have this memory recorded */

  list=BOARD_MEMORY(thisboard,mem);
  sflag=1;
  while(list && sflag) {
    if (MEMORY_READER(list) == MEMORY_READER(mboard_type) &&
	  MEMORY_TIMESTAMP(list) == MEMORY_TIMESTAMP(mboard_type)) {
      /* nope, slot, reader, and timestamp equal, so already saved */
      sflag=0;
	}
  list=MEMORY_NEXT(list);
  }

  if(sflag) {
    list=BOARD_MEMORY(thisboard,mem);
    BOARD_MEMORY(thisboard,mem) = mboard_type;
    MEMORY_NEXT(mboard_type)=list;
  } else {
      if(mboard_type) 
        free(mboard_type);
  }
        
  /* before we print out the message, we may as well restore a human
 				readable timestamp. */
  tmstr = (char *) asctime(localtime(&MESG_TIMESTAMP(message)));
  *(tmstr + strlen(tmstr) - 1) = '\0';
  sprintf(name,"%s", get_name_by_id(MESG_POSTER(message)) ? 
		get_name_by_id(MESG_POSTER(message)) : "Unknown");
  sprintf(buf,"Message %d : %6.10s (%s) :: %s\r\n\r\n%s\r\n", arg, tmstr, CAP(name),
      MESG_SUBJECT(message) ? MESG_SUBJECT(message) : "No Subject",
      MESG_DATA(message) ? MESG_DATA(message) : "Looks like this message is empty.");

  page_string(ch->desc, buf, 1);
  Board_save_board(BOARD_VNUM(thisboard));
  return 1;
}
  
ACMD(do_delete) {
  int found=0,mnum=0;
  struct obj_data *obj;
  char number[MAX_STRING_LENGTH];

  for (obj = ch->carrying; obj;obj=obj->next_content) {
    if(GET_OBJ_TYPE(obj) == ITEM_BOARD) {
      found=1;
      break;
    }
  }

  if(!obj) {
    for (obj = world[ch->in_room].contents; obj;obj=obj->next_content) {
      if(GET_OBJ_TYPE(obj) == ITEM_BOARD) {
        found=1;
        break;
      }
    }
  }

  if (obj) {
    argument = one_argument(argument, number);
    if (!*number) {
      send_to_char(ch, "Delete what?\r\n");
      return;
    }
    if (!isdigit(*number) || (!(mnum = atoi(number)))) {
      send_to_char(ch, "You must type the number of the message you wish to delete.\r\n");
      return;
    }
    Board_remove_msg(GET_OBJ_VNUM(obj), ch, mnum);
  }

  /* No board in the room? Send generic message -spl */
  if (found == 0) {
    send_to_char(ch, "Sorry, you may only delete messages posted on a board.\r\n");
  }
}


int Board_remove_msg(int board_vnum, struct char_data * ch, int arg)
{
  extern struct board_info_type *board_info;
  struct board_info_type *thisboard=board_info;
  struct board_msg_info *message, *old_message;
  struct descriptor_data *d;
  int msgcount;
  char buf[MAX_STRING_LENGTH];
 
  if(IS_NPC(ch)) {
    send_to_char(ch, "Nuts.. looks like you forgot your eraser back in mobland...\r\n");
    return 1;
  }

  while(thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard=BOARD_NEXT(thisboard);
  }

  if (!thisboard) {
    send_to_char(ch, "Error: Your board could not be found. Please report.\n");
    log("Error in Board_remove_msg - board #%d", board_vnum);
    return 0;
  }

  message=BOARD_MESSAGES(thisboard);
  if (arg < 1) {
    send_to_char(ch, "You must specify the (positive) number of the message to be read!\r\n");
    return 1;
  }

  old_message=NULL;
  for(msgcount=arg;message && msgcount!=1;msgcount--) {
    old_message=message;
    message=MESG_NEXT(message);
  }

  if(!message) {
    send_to_char(ch, "That message exists only in your imagination.\r\n");
    return 1;
  }

  if(GET_IDNUM(ch) != MESG_POSTER(message) && 
 	GET_LEVEL(ch) < REMOVE_LVL(thisboard)) {
    send_to_char(ch, "You are not holy enough to remove other people's messages.\r\n");
    return 1;
  }
  /* perform check for mesg in creation */
  for (d = descriptor_list; d; d = d->next)
    if (!d->connected && d->str == &(MESG_DATA(message))) {
      send_to_char(ch, "At least wait until the author is finished before removing it!\r\n");
      return 1;
	}
 
  /* everything else is peachy, kill the message */
  if(MESG_NEXT(message)) {
    if(old_message) {
      MESG_NEXT(old_message)=MESG_NEXT(message);
    } else {
      BOARD_MESSAGES(thisboard)=MESG_NEXT(message);
	}
  } else {
    if (old_message) {
      MESG_NEXT(old_message)=NULL;
	} else {
      BOARD_MESSAGES(thisboard)=NULL;
	}
  }
  
  free(message);
  BOARD_MNUM(thisboard)=BOARD_MNUM(thisboard)-1;
 
  send_to_char(ch, "Message removed.\r\n");
  sprintf(buf, "$n just removed message %d.", arg);
  act(buf, FALSE, ch, 0, 0, TO_ROOM);

  Board_save_board(BOARD_VNUM(thisboard));
  return 1;
}

int Board_save_board(int board_vnum) {
  extern struct board_info_type *board_info;
  struct board_info_type *thisboard=board_info;
  struct board_msg_info *message;
  struct board_memory_type *memboard;
  struct obj_data *obj;
  FILE *fl;
  int i=1,counter=0;
  char buf[MAX_STRING_LENGTH];

  while(thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard=BOARD_NEXT(thisboard);
  }
  if(!thisboard) 
    thisboard=create_new_board(board_vnum);

  /* before we save to the file, lets make sure that our board is updated */

  obj = read_object(real_object(board_vnum), REAL);
  READ_LVL(thisboard)=GET_OBJ_VAL(obj, 0);
  WRITE_LVL(thisboard)=GET_OBJ_VAL(obj, 1);
  REMOVE_LVL(thisboard)=GET_OBJ_VAL(obj, 2);
  extract_obj(obj);
 
  sprintf(buf,"%s/%d",BOARD_DIRECTORY,board_vnum);
 
  if (!(fl = fopen(buf, "wb"))) {
    perror("Error writing board");
    return 0;
  }
  /* now we write out the basic stats of the board */
  
  fprintf(fl,"Board File\n%d\n",BOARD_MNUM(thisboard));

  for(message=BOARD_MESSAGES(thisboard);message;message=MESG_NEXT(message))
  {
    fprintf(fl,"#%d\n"
               "%ld\n"
               "%ld\n"
               "%s\n"
               "%s~\n",
               i, MESG_POSTER(message), MESG_TIMESTAMP(message),
               MESG_SUBJECT(message), MESG_DATA(message));  
    i++;
  }
  /* now write out the saved info.. */
  for(counter=0;counter!=301;counter++) {
    memboard=BOARD_MEMORY(thisboard,counter);
    while(memboard) {
      fprintf(fl,"S%d %d %d\n",counter, MEMORY_READER(memboard), 
		MEMORY_TIMESTAMP(memboard));
      memboard=MEMORY_NEXT(memboard);
	}
  }
  fclose(fl);
  return 1;
}
  
void Board_write_message(int board_vnum, struct char_data *ch, char *arg){
  struct board_info_type *thisboard=board_info;
  struct board_msg_info *message,*old_message;

  /* board locate */
  if(IS_NPC(ch)) {
    send_to_char(ch, "Planian police thwart your attempt at free speech.\r\n");
    return;
  }
 
  while(thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard=BOARD_NEXT(thisboard);
  }
  if (!thisboard) {
    send_to_char(ch, "Error: Your board could not be found. Please report.\n");
    log("Error in Board_display_msg - board #%d", board_vnum);
    return;
  }
 
  if (GET_LEVEL(ch) < WRITE_LVL(thisboard)) {
    send_to_char(ch, "You are not holy enough to write on this board.\r\n");
    return;
  }

  if(!*arg || !arg)
    sprintf(arg,"No Subject");
  
  skip_spaces(&arg);
  delete_doubledollar(arg);
  
  arg[81] = '\0';
  
  CREATE(message, struct board_msg_info, 1);
  MESG_POSTER(message)=GET_IDNUM(ch);
  MESG_TIMESTAMP(message)=time(0);
  MESG_SUBJECT(message) = strdup(arg);
  MESG_NEXT(message)=NULL;
  MESG_DATA(message)=NULL;
  BOARD_MNUM(thisboard) = BOARD_MNUM(thisboard) + 1;
 
  if (BOARD_MESSAGES(thisboard)) {
    old_message=BOARD_MESSAGES(thisboard);
    BOARD_MESSAGES(thisboard)=message;
    MESG_NEXT(message)=old_message;
  } else 
    BOARD_MESSAGES(thisboard)=message;
  
  send_to_char(ch, "Write your message.\r\n");
  send_editor_help(ch->desc);
  act("$n starts to write a message.", TRUE, ch, 0, 0, TO_ROOM);
 
  if (!IS_NPC(ch))
    SET_BIT_AR(PLR_FLAGS(ch), PLR_WRITING);
  
  string_write(ch->desc, &(MESG_DATA(message)),
		MAX_MESSAGE_LENGTH, board_vnum + BOARD_MAGIC, NULL);
  
  return;
}
  
void Board_respond_message(int board_vnum, struct char_data *ch, int mnum){
  struct board_info_type *thisboard=board_info;
  struct board_msg_info *message,*old_message,*other;
  char number[MAX_STRING_LENGTH], buf[MAX_STRING_LENGTH];
  int gcount=0;

  /* board locate */
  while(thisboard) {
    if (BOARD_VNUM(thisboard) == board_vnum)
      break;
    thisboard=BOARD_NEXT(thisboard);
  }

  if (!thisboard) {
    send_to_char(ch, "Error: Your board could not be found. Please report.\n");
    log("Error in Board_display_msg - board #%d", board_vnum);
    return;
  }
 
  if (GET_LEVEL(ch) < WRITE_LVL(thisboard)) {
    send_to_char(ch, "You are not holy enough to write on this board.\r\n");
    return;
  }
  if (GET_LEVEL(ch) < READ_LVL(thisboard)) {
    send_to_char(ch, "You are not holy enough to respond to this board.\r\n");
    return;
   }
 
  if (mnum < 0 || mnum > BOARD_MNUM(thisboard)) {
    send_to_char(ch, "You can only respond to an actual message.\r\n");
    return;
  }
  
  other=BOARD_MESSAGES(thisboard);

  /*locate message to be repsponded to */
  for(gcount=0;other && gcount != (mnum-1); gcount++)
    other=MESG_NEXT(other);
  
  CREATE(message, struct board_msg_info, 1);
  MESG_POSTER(message)=GET_IDNUM(ch);
  MESG_TIMESTAMP(message)=time(0);
  sprintf(buf,"Re: %s",MESG_SUBJECT(other));
  MESG_SUBJECT(message)=strdup(buf);
  MESG_NEXT(message)=NULL;
  MESG_DATA(message)=NULL;
  BOARD_MNUM(thisboard) = BOARD_MNUM(thisboard) + 1;

  if (BOARD_MESSAGES(thisboard)) {
    old_message=BOARD_MESSAGES(thisboard);
    BOARD_MESSAGES(thisboard)=message;
    MESG_NEXT(message)=old_message;
  } else 
    BOARD_MESSAGES(thisboard)=message;
  
  send_to_char(ch, "Write your message.  (/s saves /h for help)\r\n\r\n");
  act("$n starts to write a message.", TRUE, ch, 0, 0, TO_ROOM);
 
  if (!IS_NPC(ch))
    SET_BIT_AR(PLR_FLAGS(ch), PLR_WRITING);
  
  
  /* don't need number anymore, so we'll reuse it. */
  sprintf(number,"\t&y-------&n %s wrote: &y-------&n\r\n%s\t&y-------&n End Quote &y-------&n\r\n",
	  get_name_by_id(MESG_POSTER(other)), MESG_DATA(other));
 
  MESG_DATA(message)=strdup(number);
 
  ch->desc->backstr = strdup(number);
  write_to_output(ch->desc, number);
 
  string_write(ch->desc, &(MESG_DATA(message)),
	MAX_MESSAGE_LENGTH, board_vnum + BOARD_MAGIC, NULL);
 
  return;
}
  
int mesglookup(struct board_msg_info *message,struct char_data *ch,
		struct board_info_type *board) {
 
  int mem;
  struct board_memory_type *mboard_type;
  
  mem = ((MESG_TIMESTAMP(message)%301 + MESG_POSTER(message)%301)%301);

  /* now, we check the mem slot. If its null, we return no, er.. 0..
   * if its full, we double check against the timestamp and reader - 
   * mislabled as poster, but who cares... if they're not true, we
   * go to the linked next slot, and repeat 
   */

  mboard_type=BOARD_MEMORY(board,mem);
  while(mboard_type) {
    if(MEMORY_READER(mboard_type)==GET_IDNUM(ch) &&
  	  MEMORY_TIMESTAMP(mboard_type)==MESG_TIMESTAMP(message)) {
      return 1;
    } else {
      mboard_type=MEMORY_NEXT(mboard_type);
    }
  }
  return 0;
}
  
  
ACMD(do_respond) {
  int found=0,mnum=0;
  struct obj_data *obj;
  char number[MAX_STRING_LENGTH];
   
  if(IS_NPC(ch)) {
    send_to_char(ch, "As a mob, you never bothered to learn to read or write.\r\n");
    return;
  }
  
  for (obj = ch->carrying; obj;obj=obj->next_content) {
    if(GET_OBJ_TYPE(obj) == ITEM_BOARD) {
      found=1;
      break;
    }
  }

  if(!obj) {
    for (obj = world[ch->in_room].contents; obj;obj=obj->next_content) {
      if(GET_OBJ_TYPE(obj) == ITEM_BOARD) {
        found=1;
        break;
      }
    }
  }

  if (obj) {
    argument = one_argument(argument, number);
    if (!*number) {
      send_to_char(ch, "Respond to what?\r\n");
      return;
    }
    if (!isdigit(*number) || (!(mnum = atoi(number)))) {
      send_to_char(ch, "You must type the number of the message you wish to reply to.\r\n");
      return;
    }
    Board_respond_message(GET_OBJ_VNUM(obj), ch, mnum);
  }

  /* No board in the room? Send generic message -spl */
  if (found == 0) {
    send_to_char(ch, "Sorry, you may only reply to messages posted on a board.\r\n");
  }
}

