/* ************************************************************************
*   File: boards.h                                      Part of CircleMUD *
*  Usage: header file for bulletin boards                                 *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

#define MAX_MESSAGE_LENGTH	4096	/* arbitrary -- change if needed */
  
#define BOARD_DIRECTORY "etc/boards"
#define BOARD_MAGIC	1048575	/* arbitrary number - see modify.c */
 
struct board_msg_info {
  long poster;       /* pointer to messages' poster's uid */
  time_t timestamp;    /* time stamp */
  char *subject; /* subject header */
  char *data;   /* the actual post */
  struct board_msg_info *next_message;   /* the next message! duh! */
};
  
/* sorry everyone, but inorder to allow for board memory, I had to make
    this in the following nasty way.  Sorri. */
 
struct board_memory_type {
  int timestamp;
  int reader;
  struct board_memory_type *next;
};
 
 
struct board_info_type {
  int  read_lvl;       /* min level to read messages on this board */
  int  write_lvl;      /* min level to write messages on this board */
  int  remove_lvl;     /* min level to remove messages from this board */
  int  num_messages;           /* num messages of this board */
  int  vnum;
  struct board_info_type *next_board;
  struct board_msg_info *messages;
 
  /* why 301? why not?  It might not be the greatest, but if you really
     know what a hash is, you'll realize that in this case, I didn't even
     work on the algorithm, so it shouldn't make a bit of difference */

  struct board_memory_type *memory[301];
};
 

#define READ_LVL(i) (i->read_lvl)
#define WRITE_LVL(i) (i->write_lvl)
#define REMOVE_LVL(i) (i->remove_lvl)
#define BOARD_MNUM(i) (i->num_messages)
#define BOARD_VNUM(i) (i->vnum)
#define BOARD_NEXT(i) (i->next_board)
#define BOARD_MESSAGES(i) (i->messages)
#define BOARD_MEMORY(i,j) (i->memory[j])
 
#define MESG_POSTER(i) (i->poster)
#define MESG_TIMESTAMP(i) (i->timestamp)
#define MESG_SUBJECT(i) (i->subject)
#define MESG_DATA(i) (i->data)
#define MESG_NEXT(i) (i->next_message)
 
#define MEMORY_TIMESTAMP(i) (i->timestamp)
#define MEMORY_READER(i) (i->reader)
#define MEMORY_NEXT(i) (i->next)
 
void Board_init_boards(void);
struct board_info_type * create_new_board(int board_vnum);
void parse_message( FILE *fl, struct board_info_type *t_board);
void look_at_boards(void);
int Board_show_board(int board_vnum, struct char_data *ch);
int Board_display_msg(int board_vnum, struct char_data * ch, int arg);
int Board_remove_msg(int board_vnum, struct char_data * ch, int arg);
int Board_save_board(int board_vnum);
void Board_write_message(int board_vnum, struct char_data *ch, char *arg);
void reverse_message_order(struct board_info_type *t_board);
int mesglookup(struct board_msg_info *message,struct char_data *ch,
		struct board_info_type *board);
void Board_respond_message(int board_vnum, struct char_data *ch, int mnum);
  
