/* ************************************************************************
*    File: introduction.c                                                 *
*   Usage: Source file for introduction system and memory                 *
*  Author: Mathew Earle Reuther (Corwynn) graymere@zipcon.net             *
*          (Contributors/influencers listed below.)                       *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  ApotheosisMUD (C) 2002, Mathew Earle Reuther (Corwynn)                 *
*  Copyright (C) 1993, 94 by the Trustees of the Johns Hopkins University *
*  CircleMUD is based on DikuMUD, Copyright (C) 1990, 1991.               *
************************************************************************ */

/*
 * This file was created to store all the code related to the ApotheosisMUD
 * introduction system and player memory.  It was heavily influenced (read:
 * the code was almost wholly written) by Erwin Andreasen and his work on a
 * MERC version of an intro/memory system.  Changes to make it work with
 * ApotheosisMUD were made by me (with assistance from the list) - Corwynn
 */

#include "conf.h"
#include "sysdep.h"


#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "introduction.h"


extern const char *pc_race_types_type2[];
extern const char *pc_race_types[];


/* Does ch know victim? */
bool knows_char(struct char_data *ch, struct char_data *victim)
{
  int bot, mid, top;

  /* Characters always know NPCs and IMMORTALS and vice versa */    
  if (IS_NPC(ch) || IS_NPC(victim) || (GET_LEVEL(ch) >= LVL_IMMORT) 
                 || (GET_LEVEL(victim) >= LVL_IMMORT) || ch == victim) 
    return TRUE;

    if (!ch->player_specials->memory)
      return FALSE;

  /* If the data was sorted, a binary search could be attempted */
  /* indeed - so the list is sorted */
  bot = 1;
  top = ch->player_specials->memory[0];

  /* perform binary search */

  for (;;) {
    mid = (bot + top) / 2;

    if (bot>=top)
      return FALSE;

    if (ch->player_specials->memory[mid] == GET_IDNUM(victim))
      return TRUE;

    if (ch->player_specials->memory[mid] > GET_IDNUM(victim))
      top = mid - 1;
    else
      bot = mid + 1;
  }
  return FALSE;
}



/* Add victim to ch's know list */

void add_know(struct char_data *ch, struct char_data *victim)
{
  int i, found = 0;

  if (knows_char(ch,victim))
    return;

  /* simple case first */
  if (!ch->player_specials->memory) {
    CREATE(ch->player_specials->memory, long, 2);
    ch->player_specials->memory[0] = 1;
    ch->player_specials->memory[1] = GET_IDNUM(victim);
    return;
  }

  ch->player_specials->memory[0]++;
  /* number of remembered + index -> +1*/
  RECREATE(ch->player_specials->memory, long, ch->player_specials->memory[0]+1);

  for (i = ch->player_specials->memory[0]+1;i > 1; i--) {
    if (ch->player_specials->memory[i-1] < GET_IDNUM(victim)) {
      ch->player_specials->memory[i] = GET_IDNUM(victim);
      found = i;
    } else
      ch->player_specials->memory[i] = ch->player_specials->memory[i-1];
  }

  if (!found)
    ch->player_specials->memory[1] = GET_IDNUM(victim);
}

char *display_long_name(struct char_data *ch, struct char_data *vict)
{
  if (knows_char(ch, vict) && !IS_NPC(vict))
    return(vict->player.name); /* For players */
  else if (knows_char(ch, vict) && IS_NPC(vict))
	return(vict->player.short_descr); /* For Mobiles */
  else {
    char buf[64];
    static char buf2[256];

    sprintf(buf, "%s", GET_FEATURE1(vict));

    sprintf(buf2, "%s, %s %s %s", CAP(buf), GET_FEATURE2(vict), pc_race_types_type2[(int)GET_RACE(vict)], 
    show_sex(vict));
	return(buf2);
  }
}

char *display_short_name(struct char_data *ch, struct char_data *vict)
{
  if (knows_char(ch, vict) && !IS_NPC(vict))
    return(vict->player.name); /* For players */
  else if (knows_char(ch, vict) && IS_NPC(vict))
	return(vict->player.short_descr); /* For Mobiles */
  else {
    static char buf[256];
	  
	sprintf(buf, "%s %s", GET_FEATURE1(vict), pc_race_types[(int)GET_RACE(vict)]);
	return(buf);
  }
}

char *show_sex(struct char_data *ch)
{
  switch (GET_SEX(ch)) {
  case SEX_MALE:
  case SEX_NEUTRAL:
    return("man");
  case SEX_FEMALE:
    return("woman");
  default:
	return("Oh dear. WHAT are you?!");
  }
}

void show_features(struct descriptor_data *d)
{
  char buf[64];

  sprintf(buf, "%s", GET_FEATURE1(d->character));

  write_to_output(d, 
	  "\r\n&nFeatures Confirmation Selection Menu\r\n\r\n"

      "Please take a moment to read the short description below.  When you are\r\n"
	  "positive that the description is appropriate, you may continue through\r\n"
	  "the generation process.  Your descriptions are currently:\r\n\r\n"

      "Long version &y(&nshown when you are standing in a room&y)&n:\r\n"
      "  %s, %s %s %s is standing here.\r\n\r\n"

	  "Short version &y(&nshown when you are fighting, enter a room, or gossip&y)&n:\r\n"
      "  %s %s has arrived from below.\r\n\r\n"

	  " [&yConfirmation&n]&w &c->&w (&Y1&w) Continue With These Descriptions\r\n&n"
      " [&yModification&n]&w &c->&w (&Y2&w) Back to Feature Selection Menu 1\r\n&n",

      CAP(buf), GET_FEATURE2(d->character), pc_race_types_type2[(int)GET_RACE(d->character)], 
	  show_sex(d->character), buf, pc_race_types[(int)GET_RACE(d->character)]);
}

const char *FEATURES1MENU =
" &y============================================================================&n\r\n"
"\r\n"
"            Physical Characteristics Selection Menu (Part 1)\r\n\r\n"

"This is the first of the features selection menus. Here you will choose a\r\n"
"single obvious feature to be displayed to those people who do not know who\r\n"
"you are. Your second feature will be chosen after this one.\r\n\r\n"

" [&yHair&n]&w        &c->&w (&Y1&w) Choose a Hair Feature&n\r\n"
" [&ySkin&n]&w        &c->&w (&Y2&w) Choose a Skin Feature&n\r\n"
" [&yFace&n]&w        &c->&w (&Y3&w) Choose a Facial Feature&n\r\n"
" [&yEyes&n]&w        &c->&w (&Y4&w) Choose a Eye Feature&n\r\n"
" [&ySize&n]&w        &c->&w (&Y5&w) Choose a Size Feature&n\r\n"
" [&yBody&n]&w        &c->&w (&Y6&w) Choose a Bodily Feature&n\r\n\r\n"

" &w[&Y-&w]&y              Return to the racial selection menu&n\r\n";


const char *FEATURES2MENU =
" &y============================================================================&n\r\n"
"\r\n"
"            Physical Characteristics Selection Menu (Part 2)\r\n\r\n"

"This is the second feature selection menu.  You may now choose a second\r\n"
"obvious physical feature to be displayed to those people who have not met\r\n"
"you.  This feature is shown after the first feature you selected.  We\r\n"
"&G*strongly*&n recommend not picking the same type of feature twice. If you\r\n"
"chose a hair feature, try a facial feature, etc.\r\n\r\n"

" [&yHair&n]&w        &c->&w (&Y1&w) Choose a Hair Feature&n\r\n"
" [&ySkin&n]&w        &c->&w (&Y2&w) Choose a Skin Feature&n\r\n"
" [&yFace&n]&w        &c->&w (&Y3&w) Choose a Facial Feature&n\r\n"
" [&yEyes&n]&w        &c->&w (&Y4&w) Choose a Eye Feature&n\r\n"
" [&ySize&n]&w        &c->&w (&Y5&w) Choose a Size Feature&n\r\n"
" [&yBody&n]&w        &c->&w (&Y6&w) Choose a Bodily Feature&n\r\n"

" &w[&Y-&w]&y              Return to feature selection menu 1&n\r\n";


const char *HAIRFEATURESMENU =
"\r\n"
"             Physical Characteristics Selection Menu (Hair)\r\n\r\n"

"You may choose to set any one of these features.  Make sure that the two\r\n"
"features you plan to select match well.  For example: having both long hair\r\n"
"and very long hair is redundant.  &R*Don't select them together.*&n\r\n\r\n"

" [&yHair Color 1&n]&w  &c->&w (&Ya&w)Black  (&Yb&w)Raven  (&Yc&w)Brown  (&Yd&w)Dark  (&Ye&w)Auburn  (&Yf&w)Grey\r\n&n"
" [&yHair Color 2&n]&w  &c->&w (&Yg&w)Red  (&Yh&w)Firey  (&Yi&w)Blond  (&Yj&w)Blonde  (&Yk&w)Light  (&Yl&w)Blue\r\n&n"
" [&yHair Color 3&n]&w  &c->&w (&Ym&w)Chestnut  (&Yn&w)Purple  (&Yo&w)Silver  (&Yp&w)White  (&Yq&w)Green\r\n&n"
" [&yHair Texture&n]&w  &c->&w (&Yr&w)Stringy  (&Ys&w)Curly  (&Yt&w)Straight  (&Yu&w)Wavy  (&Yv&w)Fine\r\n&n"
" [&yHair Length&w]&w   &c->&w (&Yw&w)Long  (&Yx&w)Short  (&Yy&w)Cropped  (&Yz&w)Shoulder  (&YA&w)Very Long\r\n&n"
" [&yHair Style&w]&w    &c->&w (&YB&w)Ponytail  (&YC&w)Unkempt  (&YD&w)Fancy  (&YE&w)Dirty  (&YF&w)Topknot&n\r\n"
" [&yHair Quantity&w]&w &c->&w (&YG&w)Hirsuite  (&YH&w)Bald  (&YI&w)Hairless  (&YJ&w)Thin  (&YK&w)Balding&n\r\n"
" [&yBeard Color&w]&w   &c->&w (&YL&w)Dark  (&YM&w)Grey  (&YN&w)Black  (&YO&w)Light  (&YP&w)Blond  (&YQ&w)Red&n\r\n"
" [&yBeard Texture&w]&w &c->&w (&YR&w)Bushy  (&YS&w)Mustacioed  (&YT&w)Shaven  (&YU&w)Forked&n\r\n\r\n"

" &w[&Y-&w]&y             Return to the feature selection menu&n\r\n";


const char *SKINFEATURESMENU =
"\r\n"
"             Physical Characteristics Selection Menu (Skin)\r\n\r\n"

"You may choose to set any one of these features.  Make sure that the two\r\n"
"features you plan to select match well.  For example: having both long hair\r\n"
"and very long hair is redundant.  &R*Don't select them together.*&n\r\n\r\n"

" [&ySkin Tone 1&n]&w     &c->&w (&Ya&w)Pale  (&Yb&w)White  (&Yc&w)Lily  (&Yd&w)Fair  (&Ye&w)Pallid  (&Yf&w)Albino&n\r\n"
" [&ySkin Tone 2&n]&w     &c->&w (&Yg&w)Dark  (&Yh&w)Olive  (&Yi&w)Swarthy  (&Yj&w)Black  (&Yk&w)Brown  (&Yl&w)Tan&n\r\n"
" [&ySkin Tone 3&n]&w     &c->&w (&Ym&w)Light  (&Yn&w)Ivory  (&Yo&w)Ebony  (&Yp&w)Grey  (&Yq&w)Dusky\r\n&n"
" [&ySkin Texture 1&n]&w  &c->&w (&Yr&w)Smooth  (&Ys&w)Rough  (&Yt&w)Baby  (&Yu&w)Weathered  (&Yv&w)Mottled\r\n&n"
" [&ySkin Feature 2&n]&w  &c->&w (&Yw&w)Pock-marked  (&Yx&w)Scarred  (&Yy&w)Tattooed  (&Yz&w)Burned\r\n\r\n&n"

" &w[&Y-&w]&y              Return to the feature selection menu\r\n&n";


const char *SIZEFEATURESMENU =
"\r\n"
"             Physical Characteristics Selection Menu (Size)\r\n\r\n"

"You may choose to set any one of these features.  Make sure that the two\r\n"
"features you plan to select match well.  For example: having both long hair\r\n"
"and very long hair is redundant.  &R*Don't select them together.*&n\r\n\r\n"

" [&yLarge Sized 1&n]&w &c->&w (&Ya&w)Massive  (&Yb&w)Burly  (&Yc&w)Huge  (&Yd&w)Gigantic  (&Ye&w)Big\r\n&n"
" [&yLarge Sized 2&n]&w &c->&w (&Yf&w)Large  (&Yg&w)Heavy  (&Yh&w)Fat  (&Yi&w)Flabby  (&Yj&w)Obese  (&Yk&w)Solid\r\n&n"
" [&yLarge Sized 3&n]&w &c->&w (&Yl&w)Stout  (&Ym&w)Stocky  (&Yn&w)Meaty  (&Yo&w)Beefy  (&Yp&w)Chubby\r\n&n"
" [&ySmall Sized 1&n]&w &c->&w (&Yq&w)Tiny  (&Yr&w)Slender  (&Ys&w)Skinny  (&Yt&w)Willowy  (&Yu&w)Spindly\r\n&n"
" [&ySmall Sized 2&n]&w &c->&w (&Yv&w)Thin  (&Yw&w)Small  (&Yx&w)Squat  (&Yy&w)Slight  (&Yz&w)Wizened\r\n&n"
" [&ySmall Sized 3&n]&w &c->&w (&YA&w)Lean  (&YB&w)Light  (&YC&w)Emaciated  (&YD&w)Gaunt  (&YE&w)Lithe\r\n&n"

" &w[&Y-&w]&y              Return to the feature selection menu\r\n&n";


const char *FACEFEATURESMENU =
"\r\n"
"             Physical Characteristics Selection Menu (Facial)\r\n\r\n"

"You may choose to set any one of these features.  Make sure that the two\r\n"
"features you plan to select match well.  For example: having both long hair\r\n"
"and very long hair is redundant.  &R*Don't select them together.*&n\r\n\r\n"

" [&yChin&n]&w        &c->&w (&Ya&w)Big  (&Yb&w)Cleft  (&Yc&w)Weak  (&Yd&w)Double  (&Ye&w)Chinless&n\r\n"
" [&yJaw&n]&w         &c->&w (&Yf&w)Square  (&Yg&w)Slack\r\n&n"
" [&yEars&n]&w        &c->&w (&Yh&w)Big  (&Yi&w)Torn  (&Yj&w)Pointy  (&k1&w)Furry  (&Yl&w)Clipped\r\n&n"
" [&yFace&n]&w        &c->&w (&Ym&w)Long  (&Yn&w)Stone  (&Yo&w)Square  (&Yp&w)Round  (&Yq&w)Thin  (&Yr&w)Sad\r\n&n"
" [&yNose 1&n]&w      &c->&w (&Ys&w)Big  (&Yt&w)Long  (&Yu&w)Pug  (&Yv&w)Button  (&Yw&w)Potato  (&Yx&w)Flat\r\n&n"
" [&yNose 2&n]&w      &c->&w (&Yy&w)Hook  (&Yz&w)Hawk  (&YA&w)Roman  (&YB&w)Full  (&YC&w)Thin  (&YD&w)Red&n\r\n"
" [&yMisc&n]&w        &c->&w (&YE&w)Dimples  (&YF&w)Wide Mouth  (&YG&w)Harelip  (&YH&w)Pock-faced&n\r\n"
" [&yBrow&n]&w        &c->&w (&YI&w)Thick  (&YJ&w)Bushy  (&YK&w)Browless&n\r\n\r\n"

" &w[&Y-&w]&y              Return to the feature selection menu&n\r\n";


const char *BODYFEATURESMENU =
"\r\n"
"             Physical Characteristics Selection Menu (Bodily)\r\n\r\n"

"You may choose to set any one of these features.  Make sure that the two\r\n"
"features you plan to select match well.  For example: having both long hair\r\n"
"and very long hair is redundant.  &R*Don't select them together.*&n\r\n\r\n"

" [&yHead&n]&w        &c->&w (&Ya&w)Thick  (&Yb&w)Massive  (&Yc&w)Small  (&Yd&w)Pointy  (&Ye&w)Wobbly&n\r\n"
" [&yNeck&n]&w        &c->&w (&Yf&w)Thick  (&Yg&w)None  (&Yh&w)Flabby  (&Yi&w)Scrawny\r\n&n"
" [&yChest&n]&w       &c->&w (&Yj&w)Barrel  (&Yk&w)Pigeon  (&Yl&w)Massive  (&Ym&w)Tiny\r\n&n"
" [&yBack&n]&w        &c->&w (&Yn&w)Hunched  (&Yo&w)Crooked  (&Yp&w)Straight  (&Yq&w)Strong\r\n&n"
" [&yArms&n]&w        &c->&w (&Yr&w)Long  (&Ys&w)Short  (&Yt&w)Thick  (&Yu&w)Thin  (&Yv&w)Strong\r\n&n"
" [&yLegs&n]&w        &c->&w (&Yw&w)Long  (&Yx&w)Short  (&Yy&w)Thick  (&Yz&w)Thin  (&YA&w)Bow  (&B&w)Leggy\r\n&n"
" [&yCondition&n]&w   &c->&w (&YC&w)Muscular  (&YD&w)Weak  (&YE&w)Infirm  (&YF&w)Strong  (&YG&w)Healthy\r\n&n"
" [&ySexuality&n]&w   &c->&w (&YH&w)Voluptuous  (&YI&w)Busty  (&YJ&w)Well-endowed  (&YK&w)Sensual\r\n&n"

" &w[&Y-&w]&y              Return to the feature selection menu&n\r\n";


const char *EYESFEATURESMENU =
"\r\n"
"             Physical Characteristics Selection Menu (Eyes)\r\n\r\n"

"You may choose to set any one of these features.  Make sure that the two\r\n"
"features you plan to select match well.  For example: having both long hair\r\n"
"and very long hair is redundant.  &R*Don't select them together.*&n\r\n\r\n"

" [&yEye Color 1&n]&w &c->&w (&Ya&w)Black  (&Yb&w)Brown  (&Yc&w)Dark  (&Yd&w)Grey  (&Ye&w)Slate  (&Yf&w)Green&n\r\n"
" [&yEye Color 2&n]&w &c->&w (&Yg&w)Jade  (&Yh&w)Emerald  (&Yi&w)Red  (&Yj&w)Blue  (&Yk&w)Cerulean  (&Yl&w)Ice&n\r\n"
" [&yEye Color 3&n]&w &c->&w (&Ym&w)Sapphire  (&Yn&w)Sky  (&Yo&w)Violet  (&Yp&w)Purple  (&Yq&w)Amethyst&n\r\n"
" [&yEye Color 4&n]&w &c->&w (&Yr&w)Yellow  (&Ys&w)Topaz  (&Yt&w)Golden  (&Yu&w)Silver  (&Yv&w)White&n\r\n"
" [&yEye Color 5&n]&w &c->&w (&Yw&w)Pale  (&Yx&w)Hazel  (&Yy&w)Multicolored  (&Yz&w)Milky\r\n&n"
" [&yEye Shape 1&n]&w &c->&w (&YA&w)Slanted  (&Y1&w)Squinty  (&YC&w)Round  (&YD&w)Big  (&YE&w)Wide&n\r\n"
" [&yEye Shape 2&n]&w &c->&w (&YF&w)Bug  (&YG&w)Wall  (&YH&w)Cross  (&YI&w)One  (&YJ&w)Almond&n\r\n"
" [&yEye Feature&n]&w &c->&w (&YK&w)Vibrant  (&YL&w)Sad  (&YM&w)Cruel  (&YN&w)Wise  (&YO&w)Bright&n\r\n"

" &w[&Y-&w]&y              Return to the feature selection menu&n\r\n";


char *parse_hair(char arg, int menu)
{
  if (menu == 1) {
    switch (arg) {
    case 'a': return("a black-haired");   break;
    case 'b': return("a raven-haired");   break;
    case 'c': return("a brown-haired");   break;
    case 'd': return("a dark-haired");    break;
    case 'e': return("a auburn-haired");  break;
    case 'f': return("a grey-haired");    break;
    case 'g': return("a red-headed");     break;
    case 'h': return("a firey-haired");   break;
    case 'i': return("a blond-haired");   break;
    case 'j': return("a blonde-haired");  break;
    case 'k': return("a light-haired");   break;
    case 'l': return("a blue-haired");    break;
    case 'm': return("a chestnut_haired");break;
    case 'n': return("a purple-haired");  break;
    case 'o': return("a silver-haired");  break;
    case 'p': return("a white-haired");   break;
    case 'q': return("a green-haired");   break;
    case 'r': return("a stringy-haired"); break;
    case 's': return("a curly-haired");   break;
    case 't': return("a straight-haired");break;
    case 'u': return("a wavy-haired");    break;
    case 'v': return("a fine-haired");    break;
    case 'w': return("a long-haired");    break;
    case 'x': return("a short-haired");   break;
    case 'y': return("a crop-haired");    break;
    case 'z': return("a shoulder-length-haired"); break;
    case 'A': return("a very long-haired");break;
    case 'B': return("a ponytailed");     break;
    case 'C': return("an unkempt-haired");break;
	case 'D': return("a fancy-haired");   break;
    case 'E': return("a dirty-haired");   break;
    case 'F': return("a topknotted");     break;
    case 'G': return("an hirsuite");      break;
    case 'H': return("a bald");           break;
    case 'I': return("a hairless");       break;
    case 'J': return("a thin-haired");    break;
    case 'K': return("a balding");        break;
    case 'L': return("a dark-bearded");   break;
    case 'M': return("a grey-bearded");   break;
    case 'N': return("a black-bearded");  break;
    case 'O': return("a light-bearded");  break;
    case 'P': return("a blond-bearded");  break;
    case 'Q': return("a red-bearded");    break;
    case 'R': return("a bushy-bearded");  break;
    case 'S': return("a mustacioed");     break;
    case 'T': return("a clean-shaven");   break;
    case 'U': return("a fork-bearded");   break;
    case '-': return((char*)MENU_BACK);        break;
	default:  return(NULL);               break;
    }
  } else if (menu == 2) {
    switch (arg) {
    case 'a': return("black-haired");     break;
    case 'b': return("raven-haired");     break;
    case 'c': return("brown-haired");     break;
    case 'd': return("dark-haired");      break;
    case 'e': return("auburn-haired");    break;
    case 'f': return("grey-haired");      break;
    case 'g': return("red-headed");       break;
    case 'h': return("firey-haired");     break;
    case 'i': return("blond-haired");     break;
    case 'j': return("blonde-haired");    break;
    case 'k': return("light-haired");     break;
    case 'l': return("blue-haired");      break;
    case 'm': return("chestnut-haired");  break;
    case 'n': return("purple-haired");    break;
    case 'o': return("silver-haired");    break;
    case 'p': return("white-haired");     break;
    case 'q': return("green-haired");     break;
    case 'r': return("stringy-haired");   break;
    case 's': return("curly-haired");     break;
    case 't': return("straight-haired");  break;
    case 'u': return("wavy-haired");      break;
    case 'v': return("fine-haired");      break;
    case 'w': return("long-haired");      break;
    case 'x': return("short-haired");     break;
    case 'y': return("crop-haired");      break;
    case 'z': return("shoulder-length-haired"); break;
    case 'A': return("very long-haired"); break;
    case 'B': return("ponytailed");       break;
    case 'C': return("unkempt-haired");   break;
    case 'D': return("fancy-haired");     break;
    case 'E': return("dirty-haired");     break;
    case 'F': return("topknotted");       break;
    case 'G': return("hirsuite");         break;
    case 'H': return("bald");             break;
    case 'I': return("hairless");         break;
    case 'J': return("thin-haired");      break;
    case 'K': return("balding");          break;
    case 'L': return("dark-bearded");     break;
    case 'M': return("grey-bearded");     break;
    case 'N': return("black-bearded");    break;
    case 'O': return("light-bearded");    break;
    case 'Q': return("red-bearded");      break;
    case 'R': return("bushy-bearded");    break;
    case 'S': return("mustacioed");       break;
    case 'T': return("clean-shaven");     break;
    case 'U': return("fork-bearded");     break;
	case '-': return((char*)MENU_BACK);        break;
    default:  return(NULL);               break;
	}
  } else
	return(NULL);
}

char *parse_skin(char arg, int menu)
{
  if (menu == 1) {
    switch (arg) {
    case 'a': return("a pale-skinned");   break;
    case 'b': return("a white");          break;
    case 'c': return("a lily-skinned");   break;
    case 'd': return("a fair-skinned");   break;
    case 'e': return("a pallid");         break;
    case 'f': return("an albino");        break;
    case 'g': return("a dark-skinned");   break;
    case 'h': return("an olive-skinned"); break;
    case 'i': return("a swarthy");        break;
    case 'j': return("a black");          break;
    case 'k': return("a brown");          break;
    case 'l': return("a tanned");         break;
    case 'm': return("a light-skinned");  break;
    case 'n': return("an ivory-skinned"); break;
    case 'o': return("an ebony-skinned"); break;
    case 'p': return("a grey-skinned");   break;
    case 'q': return("a dusky-skinned");  break;
    case 'r': return("a smooth-skinned"); break;
    case 's': return("a rough-skinned");  break;
    case 't': return("a baby-skinned");   break;
    case 'u': return("a weathered");      break;
    case 'v': return("a mottled-skinned");break;
    case 'w': return("a pock-marked");    break;
    case 'x': return("a scarred");        break;
    case 'y': return("a tattoed");        break;
    case 'z': return("a burned");         break;
    case '-': return((char*)MENU_BACK);        break;
    default:  return(NULL);               break;
	}
  } else if (menu == 2) {
    switch (arg) {
    case 'a': return("pale-skinned");     break;
    case 'b': return("white");            break;
    case 'c': return("lily-skinned");     break;
    case 'd': return("fair-skinned");     break;
    case 'e': return("pallid");           break;
    case 'f': return("albino");           break;
    case 'g': return("dark-skinned");     break;
    case 'h': return("olive-skinned");    break;
    case 'i': return("swarthy");          break;
    case 'j': return("black");            break;
    case 'k': return("brown");            break;
    case 'l': return("tanned");           break;
	case 'm': return("light-skinned");    break;
    case 'n': return("ivory-skinned");    break;
    case 'o': return("ebony-skinned");    break;
    case 'p': return("grey-skinned");     break;
    case 'q': return("dusky-skinned");    break;
    case 'r': return("smooth-skinned");   break;
    case 's': return("rough-skinned");    break;
    case 't': return("baby-skinned");     break;
    case 'u': return("weathered");        break;
    case 'v': return("mottled-skinned");  break;
    case 'w': return("pock-marked");      break;
    case 'x': return("scarred");          break;
    case 'y': return("tattoed");          break;
    case 'z': return("burned");           break;
	case '-': return((char*)MENU_BACK);        break;
    default:  return(NULL);               break;
    }
  } else
	return(NULL);
}

char *parse_size(char arg, int menu)
{
  if (menu == 1) {
	switch (arg) {
    case 'a': return("a massive");        break;
    case 'b': return("a burly");          break;
    case 'c': return("a huge");           break;
    case 'd': return("a gigantic");       break;
    case 'e': return("a big");            break;
    case 'f': return("a large");          break;
    case 'g': return("a heavyset");       break;
    case 'h': return("a fat");            break;
    case 'i': return("a flabby");         break;
    case 'j': return("an obese");         break;
    case 'k': return("a solid");          break;
    case 'l': return("a stout");          break;
    case 'm': return("a stocky");         break;
    case 'n': return("a meaty");          break;
    case 'o': return("a beefy");          break;
    case 'p': return("a chubby");         break;
    case 'q': return("a tiny");           break;
    case 'r': return("a slender");        break;
    case 's': return("a skinny");         break;
    case 't': return("a willowy");        break;
	case 'u': return("a spindly");        break;
    case 'v': return("a thin");           break;
    case 'w': return("a small");          break;
    case 'x': return("a squat");          break;
    case 'y': return("a slight");         break;
    case 'z': return("a wizened");        break;
    case 'A': return("a lean");           break;
    case 'B': return("a light");          break;
	case 'C': return("an emaciated");     break;
    case 'D': return("a gaunt");          break;
    case 'E': return("a lithe");          break;
	case '-': return((char*)MENU_BACK);        break;
    default:  return(NULL);               break;
	}
  } else if (menu == 2) {
	switch (arg) {
    case 'a': return("massive");          break;
    case 'b': return("burly");            break;
    case 'c': return("huge");             break;
    case 'd': return("gigantic");         break;
    case 'e': return("big");              break;
    case 'f': return("large");            break;
    case 'g': return("heavyset");         break;
    case 'h': return("fat");              break;
    case 'i': return("flabby");           break;
    case 'j': return("obese");            break;
    case 'k': return("solid");            break;
    case 'l': return("stout");            break;
    case 'm': return("stocky");           break;
    case 'n': return("meaty");            break;
    case 'o': return("beefy");            break;
    case 'p': return("chubby");           break;
    case 'q': return("tiny");             break;
    case 'r': return("slender");          break;
    case 's': return("skinny");           break;
    case 't': return("willowy");          break;
	case 'u': return("spindly");          break;
    case 'v': return("thin");             break;
    case 'w': return("small");            break;
    case 'x': return("squat");            break;
    case 'y': return("slight");           break;
    case 'z': return("wizened");          break;
    case 'A': return("lean");             break;
    case 'B': return("light");            break;
	case 'C': return("emaciated");        break;
    case 'D': return("gaunt");            break;
    case 'E': return("lithe");            break;
	case '-': return((char*)MENU_BACK);        break;
    default:  return(NULL);               break;
	} 
  } else
	return(NULL);
}

char *parse_face(char arg, int menu)
{
  if (menu == 1) {
	switch (arg) {
    case 'a': return("a big-chinned");    break;
    case 'b': return("a cleft-chinned");  break;
    case 'c': return("a weak-chinned");   break;
    case 'd': return("a double-chinned"); break;
    case 'e': return("a chinless");       break;
    case 'f': return("a square-jawed");   break;
    case 'g': return("a slack-jawed");    break;
    case 'h': return("a big-eared");      break;
    case 'i': return("a torn-eared");     break;
    case 'j': return("a pointy-eared");   break;
    case 'k': return("a furry-eared");    break;
    case 'l': return("a clipped-eared");  break;
    case 'm': return("a long-faced");     break;
    case 'n': return("a stone-faced");    break;
    case 'o': return("a square-faced");   break;
    case 'p': return("a round-faced");    break;
    case 'q': return("a thin-faced");     break;
    case 'r': return("a sad-faced");      break;
    case 's': return("a big-nosed");      break;
    case 't': return("a long-nosed");     break;
    case 'u': return("a pug-nosed");      break;
    case 'v': return("a button-nosed");   break;
    case 'w': return("a potato-nosed");   break;
    case 'x': return("a flat-nosed");     break;
    case 'y': return("a hook-nosed");     break;
    case 'z': return("a hawk-nosed");     break;
    case 'A': return("a roman-nosed");    break;
    case 'B': return("a full-nosed");     break;
    case 'C': return("a thin-nosed");     break;
    case 'D': return("a red-nosed");      break;
    case 'E': return("a dimpled");        break;
    case 'F': return("a wide-mouthed");   break;
    case 'G': return("a harelipped");     break;
    case 'H': return("a pock-faced");     break;
    case 'I': return("a thick-browed");   break;
    case 'J': return("a bushy-browed");   break;
    case 'K': return("a browless");       break;
	case '-': return((char*)MENU_BACK);        break;
    default:  return(NULL);               break;
    }
  } else if (menu == 2) {
    switch (arg) {
    case 'a': return("big-chinned");    break;
    case 'b': return("cleft-chinned");  break;
    case 'c': return("weak-chinned");   break;
    case 'd': return("double-chinned"); break;
    case 'e': return("chinless");       break;
    case 'f': return("square-jawed");   break;
    case 'g': return("slack-jawed");    break;
    case 'h': return("big-eared");      break;
    case 'i': return("torn-eared");     break;
    case 'j': return("pointy-eared");   break;
    case 'k': return("furry-eared");    break;
    case 'l': return("clipped-eared");  break;
    case 'm': return("long-faced");     break;
    case 'n': return("stone-faced");    break;
    case 'o': return("square-faced");   break;
    case 'p': return("round-faced");    break;
    case 'q': return("thin-faced");     break;
    case 'r': return("sad-faced");      break;
    case 's': return("big-nosed");      break;
    case 't': return("long-nosed");     break;
    case 'u': return("pug-nosed");      break;
    case 'v': return("button-nosed");   break;
    case 'w': return("potato-nosed");   break;
    case 'x': return("flat-nosed");     break;
    case 'y': return("hook-nosed");     break;
    case 'z': return("hawk-nosed");     break;
    case 'A': return("roman-nosed");    break;
    case 'B': return("full-nosed");     break;
    case 'C': return("thin-nosed");     break;
    case 'D': return("red-nosed");      break;
    case 'E': return("dimpled");        break;
    case 'F': return("wide-mouthed");   break;
    case 'G': return("harelipped");     break;
    case 'H': return("pock-faced");     break;
    case 'I': return("thick-browed");   break;
    case 'J': return("bushy-browed");   break;
    case 'K': return("browless");       break;
	case '-': return((char*)MENU_BACK); break;
    default:  return(NULL);             break;
    }
  } else
	return(NULL);
}

char *parse_body(char arg, int menu)
{
  if (menu == 1) {
    switch (arg) {
    case 'a': return("a thick-headed");   break;
    case 'b': return("a massive-headed"); break;
    case 'c': return("a small-headed");   break;
    case 'd': return("a pointy-headed");  break;
    case 'e': return("a wobbly-headed");  break;
    case 'f': return("a thick-necked");   break;
    case 'g': return("a no-necked");      break;
    case 'h': return("a flabby-necked");  break;
    case 'i': return("a scrawny-necked"); break; 
    case 'j': return("a barrel-chested"); break;
    case 'k': return("a pigeon-chested"); break;
    case 'l': return("a massive-chested");break;
    case 'm': return("a tiny-chested");   break;
    case 'n': return("a hunchbacked");    break;
    case 'o': return("a crookbacked");    break;
    case 'p': return("a straight-backed");break;
    case 'q': return("a strong-backed");  break;
    case 'r': return("a long-armed");     break;
    case 's': return("a short-armed");    break;
    case 't': return("a thick-armed");    break;
    case 'u': return("a thin-armed");     break;
    case 'v': return("a strong-armed");   break;
    case 'w': return("a long-legged");    break;
    case 'x': return("a short-legged");   break;
    case 'y': return("a thick-legged");   break;
    case 'z': return("a thin-legged");    break;
    case 'A': return("a bow-legged");     break;
    case 'B': return("a leggy");          break;
    case 'C': return("a muscular");       break;
    case 'D': return("a weak-looking");   break;
    case 'E': return("an infirm-looking");break;
    case 'F': return("a strong-looking"); break;
    case 'G': return("a healthy-looking");break;
    case 'H': return("a voluptuous");     break;
    case 'I': return("a busty");          break;
    case 'J': return("a well-endowed");   break;
    case 'K': return("a sensual-looking");break;
	case '-': return((char*)MENU_BACK);        break;
    default:  return(NULL);               break;
    }
  } else if (menu == 2) {
    switch (arg) {
    case 'a': return("thick-headed");   break;
    case 'b': return("massive-headed"); break;
    case 'c': return("small-headed");   break;
    case 'd': return("pointy-headed");  break;
    case 'e': return("wobbly-headed");  break;
    case 'f': return("thick-necked");   break;
    case 'g': return("no-necked");      break;
    case 'h': return("flabby-necked");  break;
    case 'i': return("scrawny-necked"); break; 
    case 'j': return("barrel-chested"); break;
    case 'k': return("pigeon-chested"); break;
    case 'l': return("massive-chested");break;
    case 'm': return("tiny-chested");   break;
    case 'n': return("hunchbacked");    break;
    case 'o': return("crookbacked");    break;
    case 'p': return("straight-backed");break;
    case 'q': return("strong-backed");  break;
    case 'r': return("long-armed");     break;
    case 's': return("short-armed");    break;
    case 't': return("thick-armed");    break;
    case 'u': return("thin-armed");     break;
    case 'v': return("strong-armed");   break;
    case 'w': return("long-legged");    break;
    case 'x': return("short-legged");   break;
    case 'y': return("thick-legged");   break;
    case 'z': return("thin-legged");    break;
    case 'A': return("bow-legged");     break;
    case 'B': return("leggy");          break;
    case 'C': return("muscular");       break;
    case 'D': return("weak-looking");   break;
    case 'E': return("infirm-looking"); break;
    case 'F': return("strong-looking"); break;
    case 'G': return("healthy-looking");break;
    case 'H': return("voluptuous");     break;
    case 'I': return("busty");          break;
    case 'J': return("well-endowed");   break;
    case 'K': return("sensual-looking");break;
	case '-': return((char*)MENU_BACK);      break;
    default:  return(NULL);             break;
    }
  } else
	return(NULL);
}

char *parse_eyes(char arg, int menu)
{ 
  if (menu == 1) {
	switch (arg) {
    case 'a': return("a black-eyed");     break;
    case 'b': return("a brown-eyed");     break;
    case 'c': return("a dark-eyed");      break;
    case 'd': return("a grey-eyed");      break;
    case 'e': return("a slate-eyed");     break;
    case 'f': return("a green-eyed");     break;
    case 'g': return("a jade-eyed");      break;
    case 'h': return("an emerald-eyed");  break;
    case 'i': return("a red-eyed");       break;
    case 'j': return("a blue-eyed");      break;
    case 'k': return("a cerulean-eyed");  break;
    case 'l': return("an ice-blue-eyed"); break;
    case 'm': return("a sapphire-eyed");  break;
    case 'n': return("a sky-blue-eyed");  break;
    case 'o': return("a violet-eyed");    break;
    case 'p': return("a purple-eyed");    break;
    case 'q': return("an amethyst-eyed"); break;
    case 'r': return("a yellow-eyed");    break;
    case 's': return("a topaz-eyed");     break;
    case 't': return("a golden-eyed");    break;
    case 'u': return("a silver-eyed");    break;
    case 'v': return("a white-eyed");     break;
    case 'w': return("a pale-eyed");      break;
    case 'x': return("a hazel-eyed");     break;
    case 'y': return("a multicolored-eyed");break;
    case 'z': return("a milky-eyed");     break;
    case 'A': return("a slant-eyed");     break;
    case 'B': return("a squinty-eyed");   break;
    case 'C': return("a round-eyed");     break;
    case 'D': return("a big-eyed");       break;
    case 'E': return("a wide-eyed");      break;
    case 'F': return("a bug-eyed");       break;
    case 'G': return("a wall-eyed");      break;
    case 'H': return("a cross-eyed");     break;
    case 'I': return("a one-eyed");       break;
    case 'J': return("an almond-shaped-eyed");break;
    case 'K': return("a vibrant-eyed");   break;
    case 'L': return("a sad-eyed");       break;
    case 'M': return("a cruel-eyed");     break;
    case 'N': return("a wise-eyed");      break;
    case 'O': return("a bright-eyed");    break;
    case '-': return((char*)MENU_BACK);        break;
	default:  return(NULL);               break;
    }
  } else if (menu == 2) {
    switch (arg) {
    case 'a': return("black-eyed");     break;
    case 'b': return("brown-eyed");     break;
    case 'c': return("dark-eyed");      break;
    case 'd': return("grey-eyed");      break;
    case 'e': return("slate-eyed");     break;
    case 'f': return("green-eyed");     break;
    case 'g': return("jade-eyed");      break;
    case 'h': return("emerald-eyed");   break;
    case 'i': return("red-eyed");       break;
    case 'j': return("blue-eyed");      break;
    case 'k': return("cerulean-eyed");  break;
    case 'l': return("ice-blue-eyed");  break;
    case 'm': return("sapphire-eyed");  break;
    case 'n': return("sky-blue-eyed");  break;
    case 'o': return("violet-eyed");    break;
    case 'p': return("purple-eyed");    break;
    case 'q': return("amethyst-eyed");  break;
    case 'r': return("yellow-eyed");    break;
    case 's': return("topaz-eyed");     break;
    case 't': return("golden-eyed");    break;
    case 'u': return("silver-eyed");    break;
    case 'v': return("white-eyed");     break;
    case 'w': return("pale-eyed");      break;
    case 'x': return("hazel-eyed");     break;
    case 'y': return("multicolored-eyed");break;
    case 'z': return("milky-eyed");     break;
    case 'A': return("slant-eyed");     break;
    case 'B': return("squinty-eyed");   break;
    case 'C': return("round-eyed");     break;
    case 'D': return("big-eyed");       break;
    case 'E': return("wide-eyed");      break;
    case 'F': return("bug-eyed");       break;
    case 'G': return("wall-eyed");      break;
    case 'H': return("cross-eyed");     break;
    case 'I': return("one-eyed");       break;
    case 'J': return("almond-shaped-eyed");break;
    case 'K': return("vibrant-eyed");   break;
    case 'L': return("sad-eyed");       break;
    case 'M': return("cruel-eyed");     break;
    case 'N': return("wise-eyed");      break;
    case 'O': return("bright-eyed");    break;
	case '-': return((char*)MENU_BACK);              break;
    default:  return(NULL);             break;
    }
  } else
	return(NULL);
}
