/*************************************************************************
*   File: poll .c                                    Part of SacrificeMUD *
*  Usage: Poll system                                                     *
*                                                                         *
*  All rights reserved.  See license.doc for complete information.        *
*                                                                         *
*  Copyright (C) 2001 by the Coding Team of SacrificeMUD                  *
*  SacrificeMUD is based on circleMUD, Copyright (C) 1993, 1994.          *
*  Original Author: Keny                                                  *
 **************************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "interpreter.h"
#include "db.h"
#include "genolc.h"
#include "oasis.h"
#include "improved-edit.h"
#include "poll.h"
#include "boards.h"

/* External data */
extern struct polls_data *polls_index;
extern void	strip_cr(char *buffer);
extern void smash_tilde(char *str);
extern char *MENU;
extern int polls_top;
extern int mini_mud;

/* function protos */
ACMD(do_oasis_pedit);
void parse_poll(FILE *fl, int virtual_nr);
int setup_new_poll(struct char_data *ch, int pollnum);
int setup_existing_poll(struct char_data *ch, int pollnum);
void pedit_states_menu(struct descriptor_data *d, int pollnum);
void save_polls_results();
void showpoll_choicemade(struct descriptor_data *d, int past_poll);
void showpoll_makechoice(struct descriptor_data *d);

/* Global vars */
int poll_nr = -1;
struct poll_looking poll_look[20];

/*------------------------------------------------------------------------*\
  Utils and exported functions.
\*------------------------------------------------------------------------*/

/* For new style oasis. */
ACMD(do_oasis_pedit)
{
  int poll_num;
  struct descriptor_data *d;
  char arg[MAX_INPUT_LENGTH];

  one_argument(argument, arg);
  
  if (!*arg) {
    poll_num = polls_top;
    if (setup_new_poll(ch, polls_top) == -1)
	  return;
  } else {
    if (!is_number(arg)) {
      send_to_char(ch, "Please supply only numerical arguments.\r\n");
      return;
    } else {
	  poll_num = atoi(argument);
      if (setup_existing_poll(ch, poll_num) == -1)
	    return;
    }
  }
  
  /* Check to make sure the poll isn't already being edited. */
  for (d = descriptor_list; d; d = d->next) {
    if (STATE(d) == CON_PEDIT) {
      if (d->olc && OLC_NUM(d) == poll_num) {
        send_to_char(ch, "That poll is currently being edited by %s.\r\n", 
          PERS(d->character, ch));
        return;
	  }
	}
  }
   
  /* Retrieve the player's descriptor. */
  d = ch->desc;
  
  /* Give the descriptor an OLC structure. */
  if (d->olc) {
    mudlog(BRF, LVL_IMMORT, TRUE, "SYSERR: do_oasis_pedit: Player already had olc structure.");
    free(d->olc);
  }
  
  /* Create the OLC structure. */
  CREATE(d->olc, struct oasis_olc_data, 1);
  
  OLC_NUM(d) = poll_num;

  STATE(d) = CON_PEDIT;
  act("$n starts editing polls.", TRUE, d->character, 0, 0, TO_ROOM);
  SET_BIT_AR(PLR_FLAGS(ch), PLR_WRITING);
  
  mudlog(BRF, LVL_IMMORT, TRUE, "OLC: %s starts editing poll #%d",
    GET_NAME(ch), poll_num);

  pedit_menu(ch->desc, poll_num);
}


/**************************************************************************
 parsing function, called from db.c 
 **************************************************************************/

/*
void poll_boot(void)
{
  FILE *fl;
  char line[READ_SIZE];
  int nr;

  if (!(fl = fopen(POLL_FILE, "r"))) {
    mudlog(BRF, LVL_IMPL, TRUE, "SYSERR: Failed to open %s.", POLL_FILE);
    return;
  } 
  
  for (;;) {
    if (!get_line(fl, line) || *line == '$') {
      fclose(fl);
      return;
    }

    if (*line == '#') {
      if (sscanf(line, "#%d", &nr) != 1) {
	    log("SYSERR: Format error in %s.", POLL_FILE);
	    fclose(fl);
		return;
	  }

	  if (!polls_index) 
		CREATE(polls_index, struct polls_data, polls_top + 1);
	  else 
		RECREATE(polls_index, struct polls_data, polls_top + 1);
	  
	  parse_poll(fl, nr);
	
	  log("   Poll #%d loaded.", nr);
	}
  }
  
  fclose(fl);
}
*/
void parse_poll(FILE *fl, int virtual_nr)
{
  int t[6];
  char line[256], buf2[MAX_STRING_LENGTH];

  snprintf(buf2, sizeof(buf2), "Poll #%d", virtual_nr);

  poll_nr++;
  polls_top++;
  polls_index[poll_nr].number = virtual_nr;
  polls_index[poll_nr].name= fread_string(fl, buf2);
  polls_index[poll_nr].desc = fread_string(fl, buf2);
  polls_index[poll_nr].first_choice = fread_string(fl, buf2);
  polls_index[poll_nr].second_choice = fread_string(fl, buf2);
  polls_index[poll_nr].third_choice = fread_string(fl, buf2);
  polls_index[poll_nr].fourth_choice = fread_string(fl, buf2);
  polls_index[poll_nr].fifth_choice = fread_string(fl, buf2);
 
  if (!get_line(fl,line) || sscanf(line, "%d %d %d %d %d %d", t, t + 1, t + 2, t + 3, t + 4, t + 5) != 6) {
    fprintf(stderr, "Format error in poll #%d\n", virtual_nr);
    exit(1);
  }  
  polls_index[poll_nr].percents[0] = t[0];
  polls_index[poll_nr].percents[1] = t[1];
  polls_index[poll_nr].percents[2] = t[2];
  polls_index[poll_nr].percents[3] = t[3];
  polls_index[poll_nr].percents[4] = t[4];
  polls_index[poll_nr].status	   = t[5];
}

/*------------------------------------------------------------------------*/

int setup_existing_poll(struct char_data *ch, int pollnum)
{
  int i = 0;
  bool node_assigned = FALSE, poll_found = FALSE;
 
  for (i=0; i<polls_top; i++) 
    if (polls_index[i].number == pollnum) {
	  poll_found = TRUE;
	  break;
    }
  
  if (!poll_found) {
    send_to_char(ch, "No poll by that number, sorry.\r\n");
    cleanup_olc(ch->desc, CLEANUP_ALL);
    return (-1);
  }

  for (i = 0; i < MAX_POLL_LOOKS; i++)
    if ((POLL_LOOK_DESC(i) == NULL) && (node_assigned == FALSE)) {
      POLL_LOOK_DESC(i) = ch->desc;
      POLL_LOOK(i) = pollnum;			/* 0 is the most recent */
      POLL_LOOK_MODE(i) = POLL_EDIT_MAIN;
      node_assigned = TRUE;
    }
 
  if (!node_assigned) {
    send_to_char(ch, "Sorry, but all editing slots are full.  Try again later.\r\n");
    cleanup_olc(ch->desc, CLEANUP_ALL);
    return -1;
  }

  return 1;
}

int setup_new_poll(struct char_data *ch, int pollnum)
{
  int i = 0;
  bool node_assigned = FALSE;
  
  if (!polls_index)
    CREATE(polls_index, struct polls_data, polls_top + 1);
  
  for (i = 0; i < MAX_POLL_LOOKS; i++)
    if ((POLL_LOOK_DESC(i) == NULL) && (node_assigned == FALSE)) {
      POLL_LOOK_DESC(i) = ch->desc;
      POLL_LOOK(i) = pollnum;			/* 0 is the most recent */
      POLL_LOOK_MODE(i) = POLL_EDIT_MAIN;
      node_assigned = TRUE;
    }
 
  if (node_assigned == FALSE) {
    send_to_char(ch, "Sorry, but all editing slots are full.  Try again later.\r\n");
    cleanup_olc(ch->desc, CLEANUP_ALL);
    return (-1);
  }
 
  polls_index[polls_top].number = pollnum;
  polls_index[polls_top].name= strdup("A brandnew Poll");
  polls_index[polls_top].desc = strdup("Description of this poll");
  polls_index[polls_top].first_choice = strdup("choice1");
  polls_index[polls_top].second_choice = strdup("choice2");
  polls_index[polls_top].third_choice = strdup("choice3");
  polls_index[polls_top].fourth_choice = strdup("choice4");
  polls_index[polls_top].fifth_choice = strdup("choice5");
 
  for (i=0; i<=4; i++)
    polls_index[polls_top].percents[i] = 0; 

  polls_index[polls_top].status = POLL_FUTURE;

  polls_top++;
  return 1;
}




/*------------------------------------------------------------------------*/

void pedit_parse(struct descriptor_data *d, char *arg)
{
  int i;
  int num = -1;
  char *oldtext = NULL;
 
  for (i = 0; i < MAX_POLL_LOOKS; i++)
    if (POLL_LOOK_DESC(i) == d) num = i;
 
  if (num == -1) return;

  switch (POLL_LOOK_MODE(num)) {
  case POLL_EDIT_MAIN:
    switch (*arg) {
 	case 'q':
 	case 'Q':
 	  POLL_LOOK_MODE(num) = POLL_EDIT_CONFIRM;
 	  write_to_output(d, "Do you wish to save this poll? [y/n]");
 	  break;
	case 'A':
	case 'a':
 	  POLL_LOOK_MODE(num) = POLL_EDIT_NAME;
 	  write_to_output(d, "\r\nEnter poll name: \r\n");
 	  break;
	case 'B':
	case 'b':
 	  POLL_LOOK_MODE(num) = POLL_EDIT_DESC;
      send_editor_help(d);
      write_to_output(d, "Enter poll description:\r\n\r\n");
      if (polls_index[POLL_LOOK(num)].desc) {
	    write_to_output(d, polls_index[POLL_LOOK(num)].desc);
	    oldtext = strdup(polls_index[POLL_LOOK(num)].desc);
	  }
      string_write(d, &polls_index[POLL_LOOK(num)].desc, MAX_MESSAGE_LENGTH, 0, oldtext);
	  break;
	case '1':
 	  POLL_LOOK_MODE(num) = POLL_EDIT_CHOICE1;
 	  write_to_output(d, "\r\nEnter first choice (leave blank if none): \r\n");
 	  break;
	case '2':
 	  POLL_LOOK_MODE(num) = POLL_EDIT_CHOICE2;
 	  write_to_output(d, "\r\nEnter second choice (leave blank if none): \r\n");
 	  break;
	case '3':
 	  POLL_LOOK_MODE(num) = POLL_EDIT_CHOICE3;
 	  write_to_output(d, "\r\nEnter third choice (leave blank if none): \r\n");
 	  break;
	case '4':
 	  POLL_LOOK_MODE(num) = POLL_EDIT_CHOICE4;
 	  write_to_output(d, "\r\nEnter fourth choice (leave blank if none): \r\n");
 	  break;
	case '5':
 	  POLL_LOOK_MODE(num) = POLL_EDIT_CHOICE5;
 	  write_to_output(d, "\r\nEnter fifth choice (leave blank if none): \r\n");
 	  break;
	case 'R':
	case 'r':
	  for(i=0; i<=4; i++)
	    polls_index[POLL_LOOK(num)].percents[i] = 0;
	  write_to_output(d, "Reseting poll... Done\r\n");
	  pedit_menu(d, POLL_LOOK(num));
	  break;
	case 'S':
	case 's':
	  pedit_states_menu(d, POLL_LOOK(num));
	  POLL_LOOK_MODE(num) = POLL_EDIT_STATE;
	  break;
 	default:
 	  write_to_output(d, "\r\nInvalid command!");
  	  pedit_menu(d, POLL_LOOK(num));
 	  break;
	}  
    break;
    
  case POLL_EDIT_NAME:
    if (!genolc_checkstring(d, arg))
      break;
    if (polls_index[POLL_LOOK(num)].name)
      free(polls_index[POLL_LOOK(num)].name);
    polls_index[POLL_LOOK(num)].name = str_udup(arg);
    POLL_LOOK_MODE(num) = POLL_EDIT_MAIN;
    pedit_menu(d, POLL_LOOK(num));     
    break;

  case POLL_EDIT_CHOICE1:
    if (!genolc_checkstring(d, arg))
      break;
    if (polls_index[POLL_LOOK(num)].first_choice)
      free(polls_index[POLL_LOOK(num)].first_choice);
    if(!*arg)
	  polls_index[POLL_LOOK(num)].first_choice = NULL;
	else
	  polls_index[POLL_LOOK(num)].first_choice = str_udup(arg);
    POLL_LOOK_MODE(num) = POLL_EDIT_MAIN;
    pedit_menu(d, POLL_LOOK(num));     
    break;

  case POLL_EDIT_CHOICE2:
    if (!genolc_checkstring(d, arg))
      break;
    if (polls_index[POLL_LOOK(num)].second_choice)
      free(polls_index[POLL_LOOK(num)].second_choice);
    if(!*arg)
	  polls_index[POLL_LOOK(num)].second_choice = NULL;
	else
	  polls_index[POLL_LOOK(num)].second_choice = str_udup(arg);
    POLL_LOOK_MODE(num) = POLL_EDIT_MAIN;
    pedit_menu(d, POLL_LOOK(num));     
    break;

  case POLL_EDIT_CHOICE3:
    if (!genolc_checkstring(d, arg))
      break;
    if (polls_index[POLL_LOOK(num)].third_choice)
      free(polls_index[POLL_LOOK(num)].third_choice);
    if(!*arg)
	  polls_index[POLL_LOOK(num)].third_choice = NULL;
	else
	  polls_index[POLL_LOOK(num)].third_choice = str_udup(arg);
    POLL_LOOK_MODE(num) = POLL_EDIT_MAIN;
    pedit_menu(d, POLL_LOOK(num));     
    break;

  case POLL_EDIT_CHOICE4:
    if (!genolc_checkstring(d, arg))
      break;
    if (polls_index[POLL_LOOK(num)].fourth_choice)
      free(polls_index[POLL_LOOK(num)].fourth_choice);
    if(!*arg)
	  polls_index[POLL_LOOK(num)].fourth_choice = NULL;
	else
	  polls_index[POLL_LOOK(num)].fourth_choice = str_udup(arg);
    POLL_LOOK_MODE(num) = POLL_EDIT_MAIN;
    pedit_menu(d, POLL_LOOK(num));     
    break;

  case POLL_EDIT_CHOICE5:
    if (!genolc_checkstring(d, arg))
      break;
    if (polls_index[POLL_LOOK(num)].fifth_choice)
      free(polls_index[POLL_LOOK(num)].fifth_choice);
    if(!*arg)
	  polls_index[POLL_LOOK(num)].fifth_choice = NULL;
	else
	  polls_index[POLL_LOOK(num)].fifth_choice = str_udup(arg);
    POLL_LOOK_MODE(num) = POLL_EDIT_MAIN;
    pedit_menu(d, POLL_LOOK(num));     
    break;

  case POLL_EDIT_DESC:
    /*
     * We will NEVER get here, we hope.
     */
    mudlog(CMP, LVL_IMMORT, FALSE, "SYSERR: Reached POLL_EDIT_DESC case in pedit_parse.");
    write_to_output(d, "Oops, in POLL_EDIT_DESC.\r\n");
    break;

  case POLL_EDIT_STATE:
    switch (*arg)     {
    case '1':
      if (polls_index[num].status == POLL_GOING)
        write_to_output(d, "This poll was the ongoing poll, so there are no more ongoing polls.\r\n");
      else 
        write_to_output(d, "Status changed to OLD POLL.\r\n");
	  polls_index[POLL_LOOK(num)].status = POLL_OLD;
	  pedit_menu(d, POLL_LOOK(num));
	  POLL_LOOK_MODE(num) = POLL_EDIT_MAIN;
	  break;
	case '2':
	  write_to_output(d, "This poll is now set as FUTURE POLL.\r\n");
	  polls_index[POLL_LOOK(num)].status = POLL_FUTURE;
	  pedit_menu(d, POLL_LOOK(num));
	  POLL_LOOK_MODE(num) = POLL_EDIT_MAIN;
	  break;
	case '3':
	  for (i=0; i<polls_top; i++)
	    if (polls_index[i].status == POLL_GOING)
	      polls_index[i].status = POLL_OLD;
	  polls_index[POLL_LOOK(num)].status = POLL_GOING;
	  write_to_output(d, "This poll is now the active one.\r\n");
	  pedit_menu(d, POLL_LOOK(num));
	  POLL_LOOK_MODE(num) = POLL_EDIT_MAIN;
	  break;
	case 'Q':
	case 'q':
	  pedit_menu(d, POLL_LOOK(num));
	  POLL_LOOK_MODE(num) = POLL_EDIT_MAIN;
	  break;
 	default:
  	  write_to_output(d, "Invalid choice!\r\nWhat do you want to do? : ");
	  pedit_states_menu(d, POLL_LOOK(num));
 	  break;
    }
    break;
 
  case POLL_EDIT_CONFIRM:
    switch (*arg) {
 	  case 'Y':
 	  case 'y':
	    cleanup_olc(d, CLEANUP_ALL);
        save_polls_results();
 	    write_to_output(d, "Polls saved.\r\n");
 	    break;
 	  case 'N':
 	  case 'n':
 	    cleanup_olc(d, CLEANUP_ALL);
 	    write_to_output(d, "Poll NOT saved.\r\n");
 	    break;
 	  default:
  	    write_to_output(d, "Invalid choice!\r\nDo you wish to save this poll?  [y/n] : ");
 	  break;
	}
    return;
  }
}

/*** Genpoll.c ***/

void save_polls_results()
{
  int i;
  FILE *fl;
    
  if (!(fl = fopen(POLL_FILE, "w"))) {
    log("SYSERR: Error writing poll file '%s'", POLL_FILE);
    return;
  }
  for (i = 0; i < polls_top; i++) 
    fprintf(fl, "#%d\n"
				"%s~\n"
		        "%s~\n" /* Description */
				"%s~\n" /* 1 */
				"%s~\n" /* 2 */
				"%s~\n" /* 3 */
				"%s~\n" /* 4 */
				"%s~\n" /* 5 */
				"%d %d %d %d %d %d\n",
			polls_index[i].number, polls_index[i].name, 
			polls_index[i].desc ? polls_index[i].desc : "No description available.", 
			polls_index[i].first_choice ? polls_index[i].first_choice : "",
			polls_index[i].second_choice ? polls_index[i].second_choice : "", 
			polls_index[i].third_choice ? polls_index[i].third_choice : "", 
			polls_index[i].fourth_choice ? polls_index[i].fourth_choice : "", 
			polls_index[i].fifth_choice ? polls_index[i].fifth_choice : "", 
			polls_index[i].percents[0], polls_index[i].percents[1],
			polls_index[i].percents[2], polls_index[i].percents[3],
			polls_index[i].percents[4], polls_index[i].status);
  fputs("$~\n", fl);
  fclose(fl);
}

/*** Other ***/
void add_percent(struct descriptor_data *d, int choice)
{
  int ongoingnum = -1, i;

  for (i=0; i<polls_top;i++) 
    if (polls_index[i].status == POLL_GOING) {
      ongoingnum = i;
	break;
    }

  if (ongoingnum == -1) {
    write_to_output(d, "You reached add_percent with no ongoing poll, bug!\r\n");
    return;
  }

  polls_index[ongoingnum].percents[choice] += 1;
  save_polls_results();
  GET_POLL_VOTE(d->character) = polls_index[ongoingnum].number;
  save_char(d->character);
  write_to_output(d, "Succesfully voted!");
  showpoll_makechoice(d);
}

void poll_disp_menu(struct descriptor_data *d)
{
  int ongoingnum = -1, i;
  bool node_assigned = FALSE;

  if (!polls_index)
	return;
  
  for (i = 0; i < MAX_POLL_LOOKS; i++)
    if ((POLL_LOOK_DESC(i) == NULL) && (node_assigned == FALSE)) {
      POLL_LOOK_DESC(i) = d;
      POLL_LOOK(i) = 0;			/* 0 is the most recent */
      POLL_LOOK_MODE(i) = POLL_LOOK_MAIN;
      node_assigned = TRUE;
    }
    
  if (node_assigned == FALSE) {
    write_to_output(d, "Sorry, but all looking slots are full.  Try again later.\r\n");
    cleanup_olc(d, CLEANUP_ALL);
    return;
  }

  STATE(d) = CON_POLL;

  for (i=0; i<polls_top;i++) 
    if (polls_index[i].status == POLL_GOING) {
      ongoingnum = i;
	  break;
    }

  if (GET_POLL_VOTE(d->character) == polls_index[ongoingnum].number) 
    showpoll_choicemade(d, ongoingnum); /* Already voted */
  else /* let them vote */
    showpoll_makechoice(d);
}

/*** Menu's ***/

void pedit_menu(struct descriptor_data *d, int pollnum)
{
  get_char_colors(d->character);
  clear_screen(d);
	
  write_to_output(d,
	  "-- Poll number : [%s%d%s]\r\n"
	  "%sA%s) Change Poll name : %s%s\r\n"
	  "%sB%s) Change Poll desc : \r\n"
	  "%s%s\r\n"
	  "%s1%s) Change Choice 1  : %s%s\r\n"
	  "%s2%s) Change Choice 2  : %s%s\r\n"
	  "%s3%s) Change Choice 3  : %s%s\r\n"
	  "%s4%s) Change Choice 4  : %s%s\r\n"
	  "%s5%s) Change Choice 5  : %s%s\r\n"
	  "\r\n"
	  "%sR%s) Reset Poll\r\n"
	  "%sS%s) Change state : %s\r\n"
	  "%sQ%s) Quit polledit and save this poll\r\n"
	  "Enter choice : ",

	  cyn, polls_index[pollnum].number, nrm,
	  grn, nrm, yel, polls_index[pollnum].name, 
	  grn, nrm, yel, polls_index[pollnum].desc,
	  grn, nrm, yel, polls_index[pollnum].first_choice ? polls_index[pollnum].first_choice : "[&RNO CHOICE&y]",
	  grn, nrm, yel, polls_index[pollnum].second_choice ? polls_index[pollnum].second_choice : "[&RNO CHOICE&y]",
	  grn, nrm, yel, polls_index[pollnum].third_choice ? polls_index[pollnum].third_choice : "[&RNO CHOICE&y]",
	  grn, nrm, yel, polls_index[pollnum].fourth_choice ? polls_index[pollnum].fourth_choice : "[&RNO CHOICE&y]",
	  grn, nrm, yel, polls_index[pollnum].fifth_choice ? polls_index[pollnum].fifth_choice : "[&RNO CHOICE&y]",
	  grn, nrm,
	  grn, nrm, (polls_index[pollnum].status == POLL_GOING)?"Activated":((polls_index[pollnum].status == POLL_OLD)?"Old poll":"A will-be poll"),
	  grn, nrm
  );
}

void pedit_states_menu(struct descriptor_data *d, int pollnum)
{
  get_char_colors(d->character);
  clear_screen(d);

  write_to_output(d, 
	  "\r\n-- Current state : [%s%s%s]\r\n"
	  "%s1%s) Set as old poll\r\n"
	  "%s2%s) Set as future poll\r\n"
	  "%s3%s) Set as ongoing poll\r\n"
	  "%sQ%s) Quit this menu and return to the Main Menu\r\n"
	  "Enter choice : ",
	  cyn, (polls_index[pollnum].status == POLL_GOING)?"Activated":((polls_index[pollnum].status == POLL_OLD)?"Old poll":"A will-be poll"), nrm,
	  grn, nrm,
	  grn, nrm,
	  grn, nrm,
	  grn, nrm
  );
}

void showpoll_choicemade(struct descriptor_data *d, int past_poll)
{
  int k, total_voices = 0, voices[5];
 
  for (k=0; k<=4; k++) 
    total_voices+=polls_index[past_poll].percents[k];

  if (total_voices == 0)
    total_voices = -1;	// we can't divide by 0, so, let's put a magic number

  for (k=0; k<=4; k++) 
    voices[k] = ((polls_index[past_poll].percents[k] * 100) / total_voices);

  write_to_output(d,
	  "\r\n&w        _________________________________________________________________&n\r\n"
	  " &w      /                                                             /    \\&n\r\n"
      " &w     | &yCurrent Poll:&c %-20s                          &w|  ()  |&n\r\n"
	  "      &w| &yYou've already voted on this poll.                          &w|\\____/&n\r\n"
	  "      &w|                                                             |&n\r\n",   
	  polls_index[past_poll].name);
  
  
  /* make the percent bars */
  if ((char *)polls_index[past_poll].first_choice) {
	write_to_output(d, "      &w| &R(&Y");
    for (k = 0; k <= 100; k += (100/30)) {
	  if (voices[0] > k) 
	    write_to_output(d, "=");
	  else 
		write_to_output(d, " ");
	}
    write_to_output(d, "&R)&n   %4d vote%s :         &w|&n\r\n" 
				       "      &w| &y%-60s&w|&n\r\n", 
					   polls_index[past_poll].percents[0], polls_index[past_poll].percents[0] != 1 ? "s" : " ",
					   (char *)polls_index[past_poll].first_choice);
  }

  if ((char *)polls_index[past_poll].second_choice) {
	write_to_output(d, "      &w| &R(&Y");
    for (k = 0; k <= 100; k += (100/30)) {
	  if (voices[1] > k) 
	    write_to_output(d, "=");
	  else 
		write_to_output(d, " ");
	}
    write_to_output(d, "&R)&n   %4d vote%s :         &w|&n\r\n" 
				       "      &w| &y%-60s&w|&n\r\n", 
					   polls_index[past_poll].percents[1], polls_index[past_poll].percents[1] != 1 ? "s" : " ",
					   (char *)polls_index[past_poll].second_choice);
  }

  if ((char *)polls_index[past_poll].third_choice) {
	write_to_output(d, "      &w| &R(&Y");
    for (k = 0; k <= 100; k += (100/30)) {
	  if (voices[2] > k) 
	    write_to_output(d, "=");
	  else 
		write_to_output(d, " ");
	}
    write_to_output(d, "&R)&n   %4d vote%s :         &w|&n\r\n" 
				       "      &w| &y%-60s&w|&n\r\n", 
					   polls_index[past_poll].percents[2], polls_index[past_poll].percents[2] != 1 ? "s" : " ",
					   (char *)polls_index[past_poll].third_choice);
  }

  if ((char *)polls_index[past_poll].fourth_choice) {
	write_to_output(d, "      &w| &R(&Y");
    for (k = 0; k <= 100; k += (100/30)) {
	  if (voices[3] > k) 
	    write_to_output(d, "=");
	  else 
		write_to_output(d, " ");
	}
    write_to_output(d, "&R)&n   %4d vote%s :         &w|&n\r\n" 
				       "      &w| &y%-60s&w|&n\r\n", 
					   polls_index[past_poll].percents[3], polls_index[past_poll].percents[3] != 1 ? "s" : " ",
					   (char *)polls_index[past_poll].fourth_choice);
  }
  
  if ((char *)polls_index[past_poll].fifth_choice) {
	write_to_output(d, "      &w| &R(&Y");
    for (k = 0; k <= 100; k += (100/30)) {
	  if (voices[4] > k) 
	    write_to_output(d, "=");
	  else 
		write_to_output(d, " ");
	}
    write_to_output(d, "&R)&n   %4d vote%s :         &w|&n\r\n" 
				       "      &w| &y%-60s&w|&n\r\n", 
					   polls_index[past_poll].percents[4], polls_index[past_poll].percents[4] != 1 ? "s" : " ",
					   (char *)polls_index[past_poll].fifth_choice);
  }

  write_to_output(d,
	  "      &w|                                                             &w|&n\r\n"
	  "      &w| &yQ&w)&c Quit Poll results and return to main menu.               &w|&n\r\n"
	  "  &w____|____________________________________________________________ |&n\r\n"
	  " &w/                                                            /    \\|&n\r\n"
	  "&w| &yTotal number of votes:&c %-3d                                 &w|  ()  |&n\r\n"
	  " &w\\____________________________________________________________\\____/&n\r\n\r\n"
	  "        &yEnter choice : &n",
	  (total_voices < 0) ? 0 : total_voices);
}

void showpoll_makechoice(struct descriptor_data *d)
{
  int ongoingnum = -1, i;

  for (i=0; i<polls_top;i++) 
    if (polls_index[i].status == POLL_GOING) {
      ongoingnum = i;
	  break;
    }

  if (ongoingnum == -1) {
    write_to_output(d, "\r\nNo ongoing poll, sorry.\r\n\r\n*** PRESS RETURN: ");
	STATE(d) = CON_RMOTD; /* :) */
	return;
  }

  if (GET_POLL_VOTE(d->character) != polls_index[ongoingnum].number) {
    GET_POLL_VOTE(d->character) = -1;  
    
	write_to_output(d,
	  "\r\n&y-->&n Current Poll: %s &y<--&n                              \r\n"
	  "                                                      \r\n"
	  "%s"    
	  "                                                      \r\n"
	  "       &w ______________________________________________________________&n\r\n"
	  "       &w/                                                         /    \\&n\r\n"
      "      &w|  &yPlease make your choice:                               &w|  ()  |&n\r\n"
	  "      &w|                                                         |\\____/&n\r\n", 
	  polls_index[ongoingnum].name, polls_index[ongoingnum].desc);

	if (polls_index[ongoingnum].first_choice) 
	  write_to_output(d, "      &w| &y1&w)&c %-53s&w|&n\r\n", polls_index[ongoingnum].first_choice);
	if (polls_index[ongoingnum].second_choice) 
	  write_to_output(d, "      &w| &y2&w)&c %-53s&w|&n\r\n", polls_index[ongoingnum].second_choice);
	if (polls_index[ongoingnum].third_choice) 
	  write_to_output(d, "      &w| &y3&w)&c %-53s&w|&n\r\n", polls_index[ongoingnum].third_choice);
	if (polls_index[ongoingnum].fourth_choice) 
	  write_to_output(d, "      &w| &y4&w)&c %-53s&w|&n\r\n", polls_index[ongoingnum].fourth_choice);
	if (polls_index[ongoingnum].fifth_choice) 
	  write_to_output(d, "      &w| &y5&w)&c %-53s&w|&n\r\n", polls_index[ongoingnum].fifth_choice);


	write_to_output(d,
	  "      &w|                                                         &w|&n\r\n"
	  "      &w| &yQ&w)&c Quit Poll without voting.                            &w|&n\r\n"
	  "  &w____|________________________________________________________ |&n\r\n"
	  " &w/                                                        /    \\|&n\r\n"
	  "&w|                                                        |  ()  |&n\r\n"
	  " &w\\________________________________________________________\\____/&n\r\n\r\n"
	  "        &yEnter choice: &n");
  } else 
    showpoll_choicemade(d, ongoingnum);
}

/*** The poll parser ***/
void parse_poll_look(struct descriptor_data *d, char *arg)
{
  int i, ongoingnum = -1, num = -1;

  for (i=0; i<polls_top;i++) 
    if (polls_index[i].status == POLL_GOING) {
      ongoingnum = i;
	  break;
    }
	
  for (i = 0; i < MAX_POLL_LOOKS; i++)
    if (POLL_LOOK_DESC(i) == d) num = i;
 
  if(num == -1) return;
 
  if (GET_POLL_VOTE(d->character) == polls_index[ongoingnum].number) 
    switch (*arg) {
    case 'q':
    case 'Q':
      cleanup_olc(d, CLEANUP_ALL);
	  write_to_output(d, "%s", MENU);
      STATE(d) = CON_MENU;
      break;
    default:
      write_to_output(d, "\r\nInvalid command!");
      showpoll_makechoice(d);
      break;
  } else
    switch (*arg) {
    case 'q':
    case 'Q':
      cleanup_olc(d, CLEANUP_ALL);
      break;
    case '1':
      if (*polls_index[ongoingnum].first_choice)
        add_percent(d, 0);
      else {
        write_to_output(d, "This choice is not available in this poll.\r\n");
        showpoll_makechoice(d);
	  }
	  break;
	case '2':
	  if (*polls_index[ongoingnum].second_choice)
	    add_percent(d, 1);
	  else {
	    write_to_output(d, "This choice is not available in this poll.\r\n");
	    showpoll_makechoice(d);
	  }
	  break;
	case '3':
	  if (*polls_index[ongoingnum].third_choice)
	    add_percent(d, 2);
	  else {
	    write_to_output(d, "This choice is not available in this poll.\r\n");
	    showpoll_makechoice(d);
	  }
	  break;
	case '4':
	  if (*polls_index[ongoingnum].fourth_choice)
	    add_percent(d, 3);
	  else {
	    write_to_output(d, "This choice is not available in this poll.\r\n");
	    showpoll_makechoice(d);
	  }
	  break;
	case '5':
	  if (*polls_index[ongoingnum].fifth_choice)
	    add_percent(d, 4);
	  else {
	    write_to_output(d, "This choice is not available in this poll.\r\n");
	    showpoll_makechoice(d);
	  }
	  break;
 	default:
 	  write_to_output(d, "\r\nInvalid command!");
  	  showpoll_makechoice(d);
 	  break;
  }
  return;
}






